@@ -87,7 +87,7 @@ void CL_ClearState (void)
 
 	//johnfitz -- cl_entities is now dynamically allocated
 	cl_max_edicts = CLAMP (MIN_EDICTS,(int)max_edicts.value,MAX_EDICTS);
-	cl_entities = Hunk_AllocName (cl_max_edicts*sizeof(entity_t), "cl_entities");
+	cl_entities = (entity_t *) Hunk_AllocName (cl_max_edicts*sizeof(entity_t), "cl_entities");
 	//johnfitz
 
 //
@@ -153,7 +153,7 @@ CL_EstablishConnection
 Host should be either "local" or a net address to be passed on
 =====================
 */
-void CL_EstablishConnection (char *host)
+void CL_EstablishConnection (const char *host)
 {
 	if (cls.state == ca_dedicated)
 		return;
@@ -171,6 +171,7 @@ void CL_EstablishConnection (char *host)
 	cls.demonum = -1;			// not in the demo loop now
 	cls.state = ca_connected;
 	cls.signon = 0;				// need all the signon messages before playing
+	MSG_WriteByte (&cls.message, clc_nop);	// NAT Fix from ProQuake
 }
 
 /*
@@ -723,7 +724,7 @@ void CL_Tracepos_f (void)
 	VectorScale(vpn, 8192.0, v);
 	TraceLine(r_refdef.vieworg, v, w);
 
-	if (Length(w) == 0)
+	if (VectorLength(w) == 0)
 		Con_Printf ("Tracepos: trace didn't hit anything\n");
 	else
 		Con_Printf ("Tracepos: (%i %i %i)\n", (int)w[0], (int)w[1], (int)w[2]);
