@@ -22,13 +22,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "quakedef.h"
 
-#define NUM_SAFE_ARGVS  7
+static char     *largv[MAX_NUM_ARGVS + 1];
+static char     argvdummy[] = " ";
 
-static char     *largv[MAX_NUM_ARGVS + NUM_SAFE_ARGVS + 1];
-static char     *argvdummy = " ";
-
-static char     *safeargvs[NUM_SAFE_ARGVS] =
-	{"-stdvid", "-nolan", "-nosound", "-nocdaudio", "-nojoy", "-nomouse", "-dibonly"};
+int             safemode;
 
 cvar_t  registered = {"registered","0"};
 cvar_t  cmdline = {"cmdline","", false, true};
@@ -152,6 +149,33 @@ void InsertLinkAfter (link_t *l, link_t *after)
 ============================================================================
 */
 
+int q_vsnprintf(char *str, size_t size, const char *format, va_list args)
+{
+	int		ret;
+
+	ret = vsnprintf_func (str, size, format, args);
+
+	if (ret < 0)
+		ret = (int)size;
+
+	if ((size_t)ret >= size)
+		str[size - 1] = '\0';
+
+	return ret;
+}
+
+int q_snprintf (char *str, size_t size, const char *format, ...)
+{
+	int		ret;
+	va_list		argptr;
+
+	va_start (argptr, format);
+	ret = q_vsnprintf (str, size, format, argptr);
+	va_end (argptr);
+
+	return ret;
+}
+
 void Q_memset (void *dest, int fill, size_t count)
 {
 	size_t		i;
@@ -275,6 +299,9 @@ int Q_strncasecmp (const char *s1, const char *s2, int n)
 {
 	int             c1, c2;
 
+	if (s1 == s2 || n <= 0)
+		return 0;
+
 	while (1)
 	{
 		c1 = *s1++;
@@ -289,16 +316,14 @@ int Q_strncasecmp (const char *s1, const char *s2, int n)
 				c1 -= ('a' - 'A');
 			if (c2 >= 'a' && c2 <= 'z')
 				c2 -= ('a' - 'A');
-			if (c1 != c2)
-				return -1;              // strings not equal
 		}
 		if (!c1)
-			return 0;               // strings are equal
-//              s1++;
-//              s2++;
+			return (c2 == 0) ? 0 : -1;
+		if (c1 < c2)
+			return -1;
+		if (c1 > c2)
+			return 1;
 	}
-
-	return -1;
 }
 
 int Q_strcasecmp (const char *s1, const char *s2)
@@ -533,7 +558,7 @@ void MSG_WriteChar (sizebuf_t *sb, int c)
 		Sys_Error ("MSG_WriteChar: range error");
 #endif
 
-	buf = SZ_GetSpace (sb, 1);
+	buf = (byte *) SZ_GetSpace (sb, 1);
 	buf[0] = c;
 }
 
@@ -546,7 +571,7 @@ void MSG_WriteByte (sizebuf_t *sb, int c)
 		Sys_Error ("MSG_WriteByte: range error");
 #endif
 
-	buf = SZ_GetSpace (sb, 1);
+	buf = (byte *) SZ_GetSpace (sb, 1);
 	buf[0] = c;
 }
 
@@ -559,7 +584,7 @@ void MSG_WriteShort (sizebuf_t *sb, int c)
 		Sys_Error ("MSG_WriteShort: range error");
 #endif
 
-	buf = SZ_GetSpace (sb, 2);
+	buf = (byte *) SZ_GetSpace (sb, 2);
 	buf[0] = c&0xff;
 	buf[1] = c>>8;
 }
@@ -568,7 +593,7 @@ void MSG_WriteLong (sizebuf_t *sb, int c)
 {
 	byte    *buf;
 
-	buf = SZ_GetSpace (sb, 4);
+	buf = (byte *) SZ_GetSpace (sb, 4);
 	buf[0] = c&0xff;
 	buf[1] = (c>>8)&0xff;
 	buf[2] = (c>>16)&0xff;
@@ -590,7 +615,7 @@ void MSG_WriteFloat (sizebuf_t *sb, float f)
 	SZ_Write (sb, &dat.l, 4);
 }
 
-void MSG_WriteString (sizebuf_t *sb, char *s)
+void MSG_WriteString (sizebuf_t *sb, const char *s)
 {
 	if (!s)
 		SZ_Write (sb, "", 1);
@@ -737,7 +762,7 @@ float MSG_ReadFloat (void)
 	return dat.f;
 }
 
-char *MSG_ReadString (void)
+const char *MSG_ReadString (void)
 {
 	static char     string[2048];
 	int             l,c;
@@ -800,7 +825,7 @@ void SZ_Alloc (sizebuf_t *buf, int startsize)
 {
 	if (startsize < 256)
 		startsize = 256;
-	buf->data = Hunk_AllocName (startsize, "sizebuf");
+	buf->data = (byte *) Hunk_AllocName (startsize, "sizebuf");
 	buf->maxsize = startsize;
 	buf->cursize = 0;
 }
@@ -842,12 +867,12 @@ void *SZ_GetSpace (sizebuf_t *buf, int length)
 	return data;
 }
 
-void SZ_Write (sizebuf_t *buf, void *data, int length)
+void SZ_Write (sizebuf_t *buf, const void *data, int length)
 {
 	Q_memcpy (SZ_GetSpace(buf,length),data,length);
 }
 
-void SZ_Print (sizebuf_t *buf, char *data)
+void SZ_Print (sizebuf_t *buf, const char *data)
 {
 	int             len;
 
@@ -869,9 +894,9 @@ void SZ_Print (sizebuf_t *buf, char *data)
 COM_SkipPath
 ============
 */
-char *COM_SkipPath (char *pathname)
+const char *COM_SkipPath (const char *pathname)
 {
-	char    *last;
+	const char *last;
 
 	last = pathname;
 	while (*pathname)
@@ -888,7 +913,7 @@ char *COM_SkipPath (char *pathname)
 COM_StripExtension
 ============
 */
-void COM_StripExtension (char *in, char *out)
+void COM_StripExtension (const char *in, char *out)
 {
 	while (*in && *in != '.')
 		*out++ = *in++;
@@ -900,7 +925,7 @@ void COM_StripExtension (char *in, char *out)
 COM_FileExtension
 ============
 */
-char *COM_FileExtension (char *in)
+const char *COM_FileExtension (const char *in)
 {
 	static char exten[8];
 	int             i;
@@ -921,9 +946,9 @@ char *COM_FileExtension (char *in)
 COM_FileBase
 ============
 */
-void COM_FileBase (char *in, char *out)
+void COM_FileBase (const char *in, char *out)
 {
-	char *s, *s2;
+	const char *s, *s2;
 
 	s = in + strlen(in) - 1;
 
@@ -949,7 +974,7 @@ void COM_FileBase (char *in, char *out)
 COM_DefaultExtension
 ==================
 */
-void COM_DefaultExtension (char *path, char *extension)
+void COM_DefaultExtension (char *path, const char *extension)
 {
 	char    *src;
 //
@@ -976,7 +1001,7 @@ COM_Parse
 Parse a token out of a string
 ==============
 */
-char *COM_Parse (char *data)
+const char *COM_Parse (const char *data)
 {
 	int             c;
 	int             len;
@@ -1038,7 +1063,8 @@ skipwhite:
 		data++;
 		len++;
 		c = *data;
-	if (c=='{' || c=='}'|| c==')'|| c=='(' || c=='\'' || c==':')
+		/* commented out the check for ':' so that ip:port works */
+		if (c=='{' || c=='}'|| c==')'|| c=='(' || c=='\'' /* || c==':' */)
 			break;
 	} while (c>32);
 
@@ -1055,7 +1081,7 @@ Returns the position (1 to argc-1) in the program's argument list
 where the given parameter apears, or 0 if not present
 ================
 */
-int COM_CheckParm (char *parm)
+int COM_CheckParm (const char *parm)
 {
 	int             i;
 
@@ -1124,7 +1150,6 @@ COM_InitArgv
 */
 void COM_InitArgv (int argc, char **argv)
 {
-	qboolean        safe;
 	int             i, j, n;
 
 // reconstitute the command line for the cmdline externally visible cvar
@@ -1148,26 +1173,13 @@ void COM_InitArgv (int argc, char **argv)
 	if (n > 0 && com_cmdline[n-1] == ' ')
 		com_cmdline[n-1] = 0; //johnfitz -- kill the trailing space
 
-	Con_Printf("\nCommand line: %s",com_cmdline);
-
-	safe = false;
+	Con_Printf("Command line: %s\n", com_cmdline);
 
 	for (com_argc=0 ; (com_argc<MAX_NUM_ARGVS) && (com_argc < argc) ; com_argc++)
 	{
 		largv[com_argc] = argv[com_argc];
 		if (!Q_strcmp ("-safe", argv[com_argc]))
-			safe = true;
-	}
-
-	if (safe)
-	{
-	// force all the safe-mode switches. Note that we reserved extra space in
-	// case we need to add these, so we don't need an overflow check
-		for (i=0 ; i<NUM_SAFE_ARGVS ; i++)
-		{
-			largv[com_argc] = safeargvs[i];
-			com_argc++;
-		}
+			safemode = 1;
 	}
 
 	largv[com_argc] = argvdummy;
@@ -1192,7 +1204,7 @@ Test_f -- johnfitz
 ================
 */
 #ifdef _DEBUG
-void Test_f (void)
+static void FitzTest_f (void)
 {
 }
 #endif
@@ -1202,7 +1214,7 @@ void Test_f (void)
 COM_Init
 ================
 */
-void COM_Init (char *basedir)
+void COM_Init (const char *basedir)
 {
 	int	i = 0x12345678;
 		/*    U N I X */
@@ -1253,7 +1265,7 @@ void COM_Init (char *basedir)
 	COM_CheckRegistered ();
 
 #ifdef _DEBUG
-	Cmd_AddCommand ("test", Test_f); //johnfitz
+	Cmd_AddCommand ("fitztest", FitzTest_f); //johnfitz
 #endif
 }
 
@@ -1262,21 +1274,34 @@ void COM_Init (char *basedir)
 ============
 va
 
-does a varargs printf into a temp buffer, so I don't need to have
-varargs versions of all text functions.
+does a varargs printf into a temp buffer. cycles between
+4 different static buffers. the number of buffers cycled
+is defined in VA_NUM_BUFFS.
 FIXME: make this buffer size safe someday
 ============
 */
-char *va(char *format, ...)
+#define	VA_NUM_BUFFS	4
+#define	VA_BUFFERLEN	1024
+
+static char *get_va_buffer(void)
+{
+	static char va_buffers[VA_NUM_BUFFS][VA_BUFFERLEN];
+	static int buffer_idx = 0;
+	buffer_idx = (buffer_idx + 1) & (VA_NUM_BUFFS - 1);
+	return va_buffers[buffer_idx];
+}
+
+char *va (const char *format, ...)
 {
 	va_list		argptr;
-	static char	string[1024];
+	char		*va_buf;
 
+	va_buf = get_va_buffer ();
 	va_start (argptr, format);
-	vsprintf (string, format, argptr);
+	q_vsnprintf (va_buf, VA_BUFFERLEN, format, argptr);
 	va_end (argptr);
 
-	return string;
+	return va_buf;
 }
 
 
@@ -1338,8 +1363,8 @@ typedef struct
 
 #define MAX_FILES_IN_PACK       2048
 
-char    com_cachedir[MAX_OSPATH];
 char    com_gamedir[MAX_OSPATH];
+char    com_basedir[MAX_OSPATH];
 
 typedef struct searchpath_s
 {
@@ -1378,7 +1403,7 @@ COM_WriteFile
 The filename will be prefixed by the current game directory
 ============
 */
-void COM_WriteFile (char *filename, void *data, int len)
+void COM_WriteFile (const char *filename, const void *data, int len)
 {
 	int             handle;
 	char    name[MAX_OSPATH];
@@ -1427,14 +1452,15 @@ Copies a file over from the net to the local cache, creating any directories
 needed.  This is for the convenience of developers using ISDN from home.
 ===========
 */
-void COM_CopyFile (char *netpath, char *cachepath)
+void COM_CopyFile (const char *netpath, const char *cachepath)
 {
 	int             in, out;
 	int             remaining, count;
 	char    buf[4096];
 
 	remaining = Sys_FileOpenRead (netpath, &in);
-	COM_CreatePath (cachepath);     // create directories up to the cache file
+	Q_strcpy (buf, cachepath);
+	COM_CreatePath (buf);     // create directories up to the cache file
 	out = Sys_FileOpenWrite (cachepath);
 
 	while (remaining)
@@ -1460,14 +1486,13 @@ Finds the file in the search path.
 Sets com_filesize and one of handle or file
 ===========
 */
-int COM_FindFile (char *filename, int *handle, FILE **file)
+int COM_FindFile (const char *filename, int *handle, FILE **file)
 {
 	searchpath_t    *search;
 	char            netpath[MAX_OSPATH];
-	char            cachepath[MAX_OSPATH];
 	pack_t          *pak;
 	int                     i;
-	int                     findtime, cachetime;
+	int              findtime;
 
 	if (file && handle)
 		Sys_Error ("COM_FindFile: both handle and file set");
@@ -1525,27 +1550,6 @@ int COM_FindFile (char *filename, int *handle, FILE **file)
 			if (findtime == -1)
 				continue;
 
-		// see if the file needs to be updated in the cache
-			if (!com_cachedir[0])
-				strcpy (cachepath, netpath);
-			else
-			{
-#if defined(_WIN32)
-				if ((strlen(netpath) < 2) || (netpath[1] != ':'))
-					sprintf (cachepath,"%s%s", com_cachedir, netpath);
-				else
-					sprintf (cachepath,"%s%s", com_cachedir, netpath+2);
-#else
-				sprintf (cachepath,"%s%s", com_cachedir, netpath);
-#endif
-
-				cachetime = Sys_FileTime (cachepath);
-
-				if (cachetime < findtime)
-					COM_CopyFile (netpath, cachepath);
-				strcpy (netpath, cachepath);
-			}
-
 		//	Sys_Printf ("FindFile: %s\n",netpath);
 			com_filesize = Sys_FileOpenRead (netpath, &i);
 			if (handle)
@@ -1580,7 +1584,7 @@ returns a handle and a length
 it may actually be inside a pak file
 ===========
 */
-int COM_OpenFile (char *filename, int *handle)
+int COM_OpenFile (const char *filename, int *handle)
 {
 	return COM_FindFile (filename, handle, NULL);
 }
@@ -1593,7 +1597,7 @@ If the requested file is inside a packfile, a new FILE * will be opened
 into the file.
 ===========
 */
-int COM_FOpenFile (char *filename, FILE **file)
+int COM_FOpenFile (const char *filename, FILE **file)
 {
 	return COM_FindFile (filename, NULL, file);
 }
@@ -1628,7 +1632,7 @@ Allways appends a 0 byte.
 cache_user_t *loadcache;
 byte    *loadbuf;
 int             loadsize;
-byte *COM_LoadFile (char *path, int usehunk)
+byte *COM_LoadFile (const char *path, int usehunk)
 {
 	int             h;
 	byte    *buf;
@@ -1646,17 +1650,17 @@ byte *COM_LoadFile (char *path, int usehunk)
 	COM_FileBase (path, base);
 
 	if (usehunk == 1)
-		buf = Hunk_AllocName (len+1, base);
+		buf = (byte *) Hunk_AllocName (len+1, base);
 	else if (usehunk == 2)
-		buf = Hunk_TempAlloc (len+1);
+		buf = (byte *) Hunk_TempAlloc (len+1);
 	else if (usehunk == 0)
-		buf = Z_Malloc (len+1);
+		buf = (byte *) Z_Malloc (len+1);
 	else if (usehunk == 3)
-		buf = Cache_Alloc (loadcache, len+1, base);
+		buf = (byte *) Cache_Alloc (loadcache, len+1, base);
 	else if (usehunk == 4)
 	{
 		if (len+1 > loadsize)
-			buf = Hunk_TempAlloc (len+1);
+			buf = (byte *) Hunk_TempAlloc (len+1);
 		else
 			buf = loadbuf;
 	}
@@ -1677,24 +1681,24 @@ byte *COM_LoadFile (char *path, int usehunk)
 	return buf;
 }
 
-byte *COM_LoadHunkFile (char *path)
+byte *COM_LoadHunkFile (const char *path)
 {
 	return COM_LoadFile (path, 1);
 }
 
-byte *COM_LoadTempFile (char *path)
+byte *COM_LoadTempFile (const char *path)
 {
 	return COM_LoadFile (path, 2);
 }
 
-void COM_LoadCacheFile (char *path, struct cache_user_s *cu)
+void COM_LoadCacheFile (const char *path, struct cache_user_s *cu)
 {
 	loadcache = cu;
 	COM_LoadFile (path, 3);
 }
 
 // uses temp hunk if larger than bufsize
-byte *COM_LoadStackFile (char *path, void *buffer, int bufsize)
+byte *COM_LoadStackFile (const char *path, void *buffer, int bufsize)
 {
 	byte    *buf;
 
@@ -1715,7 +1719,7 @@ Loads the header and directory, adding the files at the beginning
 of the list so they override previous pack files.
 =================
 */
-pack_t *COM_LoadPackFile (char *packfile)
+pack_t *COM_LoadPackFile (const char *packfile)
 {
 	dpackheader_t   header;
 	int                             i;
@@ -1744,7 +1748,7 @@ pack_t *COM_LoadPackFile (char *packfile)
 
 	//johnfitz -- dynamic gamedir loading
 	//Hunk_AllocName (numpackfiles * sizeof(packfile_t), "packfile");
-	newfiles = Z_Malloc(numpackfiles * sizeof(packfile_t));
+	newfiles = (packfile_t *) Z_Malloc(numpackfiles * sizeof(packfile_t));
 	//johnfitz
 
 	Sys_FileSeek (packhandle, header.dirofs);
@@ -1767,7 +1771,7 @@ pack_t *COM_LoadPackFile (char *packfile)
 
 	//johnfitz -- dynamic gamedir loading
 	//pack = Hunk_Alloc (sizeof (pack_t));
-	pack = Z_Malloc (sizeof (pack_t));
+	pack = (pack_t *) Z_Malloc (sizeof (pack_t));
 	//johnfitz
 
 	strcpy (pack->filename, packfile);
@@ -1784,7 +1788,7 @@ pack_t *COM_LoadPackFile (char *packfile)
 COM_AddGameDirectory -- johnfitz -- modified based on topaz's tutorial
 =================
 */
-void COM_AddGameDirectory (char *dir)
+void COM_AddGameDirectory (const char *dir)
 {
 	int i;
 	searchpath_t *search;
@@ -1794,7 +1798,7 @@ void COM_AddGameDirectory (char *dir)
 	strcpy (com_gamedir, dir);
 
 	// add the directory to the search path
-	search = Z_Malloc(sizeof(searchpath_t));
+	search = (searchpath_t *) Z_Malloc(sizeof(searchpath_t));
 	strcpy (search->filename, dir);
 	search->next = com_searchpaths;
 	com_searchpaths = search;
@@ -1806,7 +1810,7 @@ void COM_AddGameDirectory (char *dir)
 		pak = COM_LoadPackFile (pakfile);
 		if (!pak)
 			break;
-		search = Z_Malloc(sizeof(searchpath_t));
+		search = (searchpath_t *) Z_Malloc(sizeof(searchpath_t));
 		search->pack = pak;
 		search->next = com_searchpaths;
 		com_searchpaths = search;
@@ -1841,41 +1845,27 @@ static void kill_id1_conback (void)  /* QuakeSpasm customization: */
 COM_InitFilesystem
 =================
 */
-void COM_InitFilesystem () //johnfitz -- modified based on topaz's tutorial
+void COM_InitFilesystem (void) //johnfitz -- modified based on topaz's tutorial
 {
 	int i, j;
-	char basedir[MAX_OSPATH];
 	searchpath_t *search;
 
 	i = COM_CheckParm ("-basedir");
 	if (i && i < com_argc-1)
-		strcpy (basedir, com_argv[i+1]);
+		strcpy (com_basedir, com_argv[i + 1]);
 	else
-		strcpy (basedir, host_parms.basedir);
+		strcpy (com_basedir, host_parms.basedir);
 
-	j = strlen (basedir);
+	j = strlen (com_basedir);
 	if (j > 0)
 	{
-		if ((basedir[j-1] == '\\') || (basedir[j-1] == '/'))
-		basedir[j-1] = 0;
+		if ((com_basedir[j-1] == '\\') || (com_basedir[j-1] == '/'))
+			com_basedir[j-1] = 0;
 	}
 
-	i = COM_CheckParm ("-cachedir");
-	if (i && i < com_argc-1)
-	{
-		if (com_argv[i+1][0] == '-')
-			com_cachedir[0] = 0;
-		else
-			strcpy (com_cachedir, com_argv[i+1]);
-	}
-	else if (host_parms.cachedir)
-		strcpy (com_cachedir, host_parms.cachedir);
-	else
-		com_cachedir[0] = 0;
-
 	// start up with GAMENAME by default (id1)
-	COM_AddGameDirectory (va("%s/"GAMENAME, basedir) );
-	strcpy (com_gamedir, va("%s/"GAMENAME, basedir));
+	COM_AddGameDirectory (va("%s/"GAMENAME, com_basedir));
+	strcpy (com_gamedir, va("%s/"GAMENAME, com_basedir));
 
 	if (!fitzmode)
 	{ /* QuakeSpasm customization: */
@@ -1887,17 +1877,17 @@ void COM_InitFilesystem () //johnfitz -- modified based on topaz's tutorial
 	com_nummissionpacks = 0;
 	if (COM_CheckParm ("-rogue"))
 	{
-		COM_AddGameDirectory (va("%s/rogue", basedir) );
+		COM_AddGameDirectory (va("%s/rogue", com_basedir));
 		com_nummissionpacks++;
 	}
 	if (COM_CheckParm ("-hipnotic"))
 	{
-		COM_AddGameDirectory (va("%s/hipnotic", basedir) );
+		COM_AddGameDirectory (va("%s/hipnotic", com_basedir));
 		com_nummissionpacks++;
 	}
 	if (COM_CheckParm ("-quoth"))
 	{
-		COM_AddGameDirectory (va("%s/quoth", basedir) );
+		COM_AddGameDirectory (va("%s/quoth", com_basedir));
 		com_nummissionpacks++;
 	}
 	//johnfitz
@@ -1906,7 +1896,7 @@ void COM_InitFilesystem () //johnfitz -- modified based on topaz's tutorial
 	if (i && i < com_argc-1)
 	{
 		com_modified = true;
-		COM_AddGameDirectory (va("%s/%s", basedir, com_argv[i+1]));
+		COM_AddGameDirectory (va("%s/%s", com_basedir, com_argv[i + 1]));
 	}
 
 	i = COM_CheckParm ("-path");
@@ -1918,7 +1908,7 @@ void COM_InitFilesystem () //johnfitz -- modified based on topaz's tutorial
 		{
 			if (!com_argv[i] || com_argv[i][0] == '+' || com_argv[i][0] == '-')
 				break;
-			search = Hunk_Alloc (sizeof(searchpath_t));
+			search = (searchpath_t *) Hunk_Alloc (sizeof(searchpath_t));
 			if (!strcmp(COM_FileExtension(com_argv[i]), "pak") )
 			{
 				search->pack = COM_LoadPackFile (com_argv[i]);
@@ -1936,4 +1926,3 @@ void COM_InitFilesystem () //johnfitz -- modified based on topaz's tutorial
 		proghack = true;
 }
 
-
