@@ -20,13 +20,15 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 // console.c
 
-#ifdef NeXT
-#include <libc.h>
-#endif
-#ifndef _MSC_VER
+#include <sys/types.h>
+#include <time.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#ifdef _WIN32
+#include <io.h>
+#else
 #include <unistd.h>
 #endif
-#include <fcntl.h>
 #include "quakedef.h"
 
 int 		con_linewidth;
@@ -43,7 +45,7 @@ int			con_totallines;		// total lines in console scrollback
 int			con_backscroll;		// lines up from bottom to display
 int			con_current;		// where next message will be printed
 int			con_x;				// offset in current line for next print
-char		*con_text=0;
+char		*con_text = NULL;
 
 cvar_t		con_notifytime = {"con_notifytime","3"};		//seconds
 cvar_t		con_logcenterprint = {"con_logcenterprint", "1"}; //johnfitz
@@ -56,9 +58,8 @@ float		con_times[NUM_CON_TIMES];	// realtime time the line was generated
 
 int			con_vislines;
 
-qboolean	con_debuglog;
+qboolean	con_debuglog = false;
 
-#define		MAXCMDLINE	256
 extern	char	key_lines[32][MAXCMDLINE];
 extern	int		edit_line;
 extern	int		key_linepos;
@@ -74,7 +75,7 @@ Con_Quakebar -- johnfitz -- returns a bar of the desired length, but never wider
 includes a newline, unless len >= con_linewidth.
 ================
 */
-char *Con_Quakebar (int len)
+const char *Con_Quakebar (int len)
 {
 	static char bar[42];
 	int i;
@@ -153,7 +154,7 @@ Con_Dump_f -- johnfitz -- adapted from quake2 source
 void Con_Dump_f (void)
 {
 	int		l, x;
-	char	*line;
+	const char	*line;
 	FILE	*f;
 	char	buffer[1024];
 	char	name[MAX_OSPATH];
@@ -187,7 +188,7 @@ void Con_Dump_f (void)
 	f = fopen (name, "w");
 	if (!f)
 	{
-		Con_Printf ("ERROR: couldn't open file.\n", name);
+		Con_Printf ("ERROR: couldn't open file %s.\n", name);
 		return;
 	}
 
@@ -298,7 +299,7 @@ void Con_CheckResize (void)
 		numchars = con_linewidth;
 
 	mark = Hunk_LowMark (); //johnfitz
-	tbuf = Hunk_Alloc (con_buffersize); //johnfitz
+	tbuf = (char *) Hunk_Alloc (con_buffersize); //johnfitz
 
 	Q_memcpy (tbuf, con_text, con_buffersize);//johnfitz -- con_buffersize replaces CON_TEXTSIZE
 	Q_memset (con_text, ' ', con_buffersize);//johnfitz -- con_buffersize replaces CON_TEXTSIZE
@@ -328,21 +329,6 @@ Con_Init
 */
 void Con_Init (void)
 {
-#define MAXGAMEDIRLEN	1000
-	char	temp[MAXGAMEDIRLEN+1];
-	char	*t2 = "/qconsole.log";
-
-	con_debuglog = COM_CheckParm("-condebug");
-
-	if (con_debuglog)
-	{
-		if (strlen (com_gamedir) < (MAXGAMEDIRLEN - strlen (t2)))
-		{
-			sprintf (temp, "%s%s", com_gamedir, t2);
-			unlink (temp);
-		}
-	}
-
 	//johnfitz -- user settable console buffer size
 	if (COM_CheckParm("-consize"))
 		con_buffersize = max(CON_MINSIZE,Q_atoi(com_argv[COM_CheckParm("-consize")+1])*1024);
@@ -350,7 +336,7 @@ void Con_Init (void)
 		con_buffersize = CON_TEXTSIZE;
 	//johnfitz
 
-	con_text = Hunk_AllocName (con_buffersize, "context");//johnfitz -- con_buffersize replaces CON_TEXTSIZE
+	con_text = (char *) Hunk_AllocName (con_buffersize, "context");//johnfitz -- con_buffersize replaces CON_TEXTSIZE
 	Q_memset (con_text, ' ', con_buffersize);//johnfitz -- con_buffersize replaces CON_TEXTSIZE
 	con_linewidth = -1;
 
@@ -407,7 +393,7 @@ All console printing must go through this in order to be logged to disk
 If no console is visible, the notify window will pop up.
 ================
 */
-void Con_Print (char *txt)
+void Con_Print (const char *txt)
 {
 	int		y;
 	int		c, l;
@@ -484,23 +470,22 @@ void Con_Print (char *txt)
 }
 
 
+// borrowed from uhexen2 by S.A. for new procs, LOG_Init, LOG_Close
+
+static char	logfilename[MAX_OSPATH];	// current logfile name
+static int	log_fd = -1;			// log file descriptor
+
 /*
 ================
 Con_DebugLog
 ================
 */
-void Con_DebugLog(char *file, char *fmt, ...)
+void Con_DebugLog(const char *msg)
 {
-    va_list argptr;
-    static char data[1024];
-    int fd;
+	if (log_fd == -1)
+		return;
 
-    va_start(argptr, fmt);
-    vsprintf(data, fmt, argptr);
-    va_end(argptr);
-    fd = open(file, O_WRONLY | O_CREAT | O_APPEND, 0666);
-    write(fd, data, strlen(data));
-    close(fd);
+	write(log_fd, msg, strlen(msg));
 }
 
 
@@ -512,15 +497,14 @@ Handles cursor positioning, line wrapping, etc
 ================
 */
 #define	MAXPRINTMSG	4096
-// FIXME: make a buffer size safe vsprintf?
-void Con_Printf (char *fmt, ...)
+void Con_Printf (const char *fmt, ...)
 {
 	va_list		argptr;
 	char		msg[MAXPRINTMSG];
 	static qboolean	inupdate;
 
 	va_start (argptr,fmt);
-	vsprintf (msg,fmt,argptr);
+	q_vsnprintf (msg, sizeof(msg), fmt, argptr);
 	va_end (argptr);
 
 // also echo to debugging console
@@ -528,7 +512,7 @@ void Con_Printf (char *fmt, ...)
 
 // log all messages to file
 	if (con_debuglog)
-		Con_DebugLog(va("%s/qconsole.log",com_gamedir), "%s", msg);
+		Con_DebugLog(msg);
 
 	if (!con_initialized)
 		return;
@@ -558,13 +542,13 @@ void Con_Printf (char *fmt, ...)
 Con_Warning -- johnfitz -- prints a warning to the console
 ================
 */
-void Con_Warning (char *fmt, ...)
+void Con_Warning (const char *fmt, ...)
 {
 	va_list		argptr;
 	char		msg[MAXPRINTMSG];
 
 	va_start (argptr,fmt);
-	vsprintf (msg,fmt,argptr);
+	q_vsnprintf (msg, sizeof(msg), fmt, argptr);
 	va_end (argptr);
 
 	Con_SafePrintf ("\x02Warning: ");
@@ -578,7 +562,7 @@ Con_DPrintf
 A Con_Printf that only shows up if the "developer" cvar is set
 ================
 */
-void Con_DPrintf (char *fmt, ...)
+void Con_DPrintf (const char *fmt, ...)
 {
 	va_list		argptr;
 	char		msg[MAXPRINTMSG];
@@ -587,7 +571,7 @@ void Con_DPrintf (char *fmt, ...)
 		return;			// don't confuse non-developers with techie stuff...
 
 	va_start (argptr,fmt);
-	vsprintf (msg,fmt,argptr);
+	q_vsnprintf (msg, sizeof(msg), fmt, argptr);
 	va_end (argptr);
 
 	Con_SafePrintf ("%s", msg); //johnfitz -- was Con_Printf
@@ -600,7 +584,7 @@ Con_DPrintf2 -- johnfitz -- only prints if "developer" >= 2
 currently not used
 ================
 */
-void Con_DPrintf2 (char *fmt, ...)
+void Con_DPrintf2 (const char *fmt, ...)
 {
 	va_list		argptr;
 	char		msg[MAXPRINTMSG];
@@ -608,7 +592,7 @@ void Con_DPrintf2 (char *fmt, ...)
 	if (developer.value >= 2)
 	{
 		va_start (argptr,fmt);
-		vsprintf (msg,fmt,argptr);
+		q_vsnprintf (msg, sizeof(msg), fmt, argptr);
 		va_end (argptr);
 
 		Con_Printf ("%s", msg);
@@ -623,14 +607,14 @@ Con_SafePrintf
 Okay to call even when the screen can't be updated
 ==================
 */
-void Con_SafePrintf (char *fmt, ...)
+void Con_SafePrintf (const char *fmt, ...)
 {
 	va_list		argptr;
 	char		msg[1024];
 	int			temp;
 
 	va_start (argptr,fmt);
-	vsprintf (msg,fmt,argptr);
+	q_vsnprintf (msg, sizeof(msg), fmt, argptr);
 	va_end (argptr);
 
 	temp = scr_disabled_for_loading;
@@ -644,7 +628,8 @@ void Con_SafePrintf (char *fmt, ...)
 Con_CenterPrintf -- johnfitz -- pad each line with spaces to make it appear centered
 ================
 */
-void Con_CenterPrintf (int linewidth, char *fmt, ...)
+void Con_CenterPrintf (int linewidth, const char *fmt, ...) __attribute__((__format__(__printf__,2,3)));
+void Con_CenterPrintf (int linewidth, const char *fmt, ...)
 {
 	va_list	argptr;
 	char	msg[MAXPRINTMSG]; //the original message
@@ -654,7 +639,7 @@ void Con_CenterPrintf (int linewidth, char *fmt, ...)
 	int		len, s;
 
 	va_start (argptr,fmt);
-	vsprintf (msg,fmt,argptr);
+	q_vsnprintf (msg, sizeof(msg), fmt, argptr);
 	va_end (argptr);
 
 	linewidth = min (linewidth, con_linewidth);
@@ -685,7 +670,7 @@ void Con_CenterPrintf (int linewidth, char *fmt, ...)
 Con_LogCenterPrint -- johnfitz -- echo centerprint message to the console
 ==================
 */
-void Con_LogCenterPrint (char *str)
+void Con_LogCenterPrint (const char *str)
 {
 	if (!strcmp(str, con_lastcenterstring))
 		return; //ignore duplicates
@@ -717,8 +702,8 @@ void Con_LogCenterPrint (char *str)
 char key_tabpartial[MAXCMDLINE];
 typedef struct tab_s
 {
-	char			*name;
-	char			*type;
+	const char	*name;
+	const char	*type;
 	struct tab_s	*next;
 	struct tab_s	*prev;
 } tab_t;
@@ -729,8 +714,8 @@ extern qboolean	keydown[256];
 typedef struct cmd_function_s
 {
 	struct cmd_function_s	*next;
-	char					*name;
-	xcommand_t				function;
+	const char		*name;
+	xcommand_t		function;
 } cmd_function_t;
 extern	cmd_function_t	*cmd_functions;
 #define	MAX_ALIAS_NAME	32
@@ -756,10 +741,11 @@ static char	bash_partial[80];
 static qboolean	bash_singlematch;
 static qboolean	map_singlematch;
 
-void AddToTabList (char *name, char *type)
+void AddToTabList (const char *name, const char *type)
 {
 	tab_t	*t,*insert;
-	char	*i_bash, *i_name;
+	char	*i_bash;
+	const char *i_name;
 
 	if (!*bash_partial)
 	{
@@ -780,7 +766,7 @@ void AddToTabList (char *name, char *type)
 		*i_bash = 0;
 	}
 
-	t = Hunk_Alloc(sizeof(tab_t));
+	t = (tab_t *) Hunk_Alloc(sizeof(tab_t));
 	t->name = name;
 	t->type = type;
 
@@ -818,7 +804,7 @@ void AddToTabList (char *name, char *type)
 // This is redefined from host_cmd.c
 typedef struct extralevel_s
 {
-	char				name[32];
+	char			name[32];
 	struct extralevel_s	*next;
 } extralevel_t;
 
@@ -829,23 +815,24 @@ extern extralevel_t	*extralevels;
 BuildMap -- stevenaaus
 ============
 */
-char *BuildMapList (char *partial)
+const char *BuildMapList (const char *partial)
 {
-	int i , match, plen;
 	static char matched[80];
 	char *i_matched, *i_name;
 	extralevel_t	*level;
+	int   init, match, plen;
 
 	memset(matched, 0, 80);
 	plen = strlen(partial);
 	match = 0;
 
-	for (level = extralevels, i = 0; level; level = level->next, i++)
+	for (level = extralevels, init = 0; level; level = level->next)
 	{
 		if (!strncmp(level->name, partial, plen))
 		{
-			if (!*matched)
+			if (init == 0)
 			{
+				init = 1;
 				strncpy (matched, level->name, 79);
 				matched[79] = '\0';
 			}
@@ -868,7 +855,7 @@ char *BuildMapList (char *partial)
 
 	if (match > 1)
 	{
-		for (level = extralevels, i = 0; level; level = level->next, i++)
+		for (level = extralevels; level; level = level->next)
 		{
 			if (!strncmp(level->name, partial, plen))
 				Con_SafePrintf ("   %s\n", level->name);
@@ -884,11 +871,11 @@ char *BuildMapList (char *partial)
 BuildTabList -- johnfitz
 ============
 */
-void BuildTabList (char *partial)
+void BuildTabList (const char *partial)
 {
 	cmdalias_t		*alias;
 	cvar_t			*cvar;
-	cmd_function_t	*cmd;
+	cmd_function_t		*cmd;
 	int				len;
 
 	tablist = NULL;
@@ -918,7 +905,7 @@ Con_TabComplete -- johnfitz
 void Con_TabComplete (void)
 {
 	char		partial[MAXCMDLINE];
-	char		*match;
+	const char	*match;
 	static char	*c;
 	tab_t		*t;
 	int			mark, i;
@@ -934,7 +921,7 @@ void Con_TabComplete (void)
 		c = key_lines[edit_line] + key_linepos - 1; //start one space left of cursor
 		while (*c!=' ' && *c!='\"' && *c!=';' && c!=key_lines[edit_line])
 			c--;
-		c++; //start 1 char after the seperator we just found
+		c++; //start 1 char after the separator we just found
 	}
 	for (i = 0; c + i < key_lines[edit_line] + key_linepos; i++)
 		partial[i] = c[i];
@@ -945,18 +932,20 @@ void Con_TabComplete (void)
 	if (!strncmp (key_lines[edit_line] + 1, "map ",4) ||
 	    !strncmp (key_lines[edit_line] + 1, "changelevel ", 12))
 	{
-		char *matched_map = BuildMapList(partial);
+		const char *matched_map = BuildMapList(partial);
 		if (!*matched_map)
 			return;
 		Q_strcpy (partial, matched_map);
 		Q_strcpy (c, partial);
 		key_linepos = c - key_lines[edit_line] + Q_strlen(matched_map); //set new cursor position
+		// if only one match, append a space
 		if ((key_lines[edit_line][key_linepos] == 0) && map_singlematch)
 		{
 			key_lines[edit_line][key_linepos] = ' ';
 			key_linepos++;
 			key_lines[edit_line][key_linepos] = 0;
 		}
+		c = key_lines[edit_line] + key_linepos;
 		return;
 	}
 
@@ -1014,7 +1003,7 @@ void Con_TabComplete (void)
 		//use prev or next to find next match
 		match = keydown[K_SHIFT] ? t->prev->name : t->next->name;
 	}
-	Hunk_FreeToLowMark(mark); //it's okay to free it here becuase match is a pointer to persistent data
+	Hunk_FreeToLowMark(mark); //it's okay to free it here because match is a pointer to persistent data
 
 // insert new match into edit line
 	Q_strcpy (partial, match); //first copy match string
@@ -1053,11 +1042,11 @@ Draws the last few lines of output transparently over the game top
 */
 void Con_DrawNotify (void)
 {
-	int		x, v;
+	int	x, v;
 	char	*text;
-	int		i;
+	int	i;
 	float	time;
-	extern char chat_buffer[];
+	extern	char chat_buffer[];
 
 	GL_SetCanvas (CANVAS_CONSOLE); //johnfitz
 	v = vid.conheight; //johnfitz
@@ -1084,30 +1073,39 @@ void Con_DrawNotify (void)
 		scr_tileclear_updates = 0; //johnfitz
 	}
 
-
 	if (key_dest == key_message)
 	{
-		char *say_prompt; //johnfitz
+		// modified by S.A to support longer lines
+
+		char	c[MAXCMDLINE+1]; // extra space == +1
+		const char	*say_prompt;
+		int	say_length, len;
 
 		clearnotify = 0;
-
 		x = 0;
 
-		//johnfitz -- distinguish say and say_team
 		if (team_message)
 			say_prompt = "say_team:";
 		else
 			say_prompt = "say:";
-		//johnfitz
+
+		say_length = strlen(say_prompt);
 
 		Draw_String (8, v, say_prompt); //johnfitz
 
-		while(chat_buffer[x])
+		text = strcpy(c, chat_buffer);
+		len  = strlen(chat_buffer);
+		text[len] = ' ';
+		text[len+1] = 0;
+		if (len >= con_linewidth - say_length)
+			text += 1 + len + say_length - con_linewidth;
+
+		while(*text)
 		{
-			Draw_Character ( (x+strlen(say_prompt)+2)<<3, v, chat_buffer[x]); //johnfitz
-			x++;
+			Draw_Character ( (x+say_length+2)<<3, v, *text); //johnfitz
+			x++; text++;
 		}
-		Draw_Character ( (x+strlen(say_prompt)+2)<<3, v, 10+((int)(realtime*con_cursorspeed)&1)); //johnfitz
+		Draw_Character ( (x+say_length+1)<<3, v, 10+((int)(realtime*con_cursorspeed)&1)); 
 		v += 8;
 
 		scr_tileclear_updates = 0; //johnfitz
@@ -1123,11 +1121,11 @@ The input line scrolls horizontally if typing goes beyond the right edge
 */
 void Con_DrawInput (void)
 {
-	extern qpic_t *pic_ovr, *pic_ins; //johnfitz -- new cursor handling
-	extern double key_blinktime;
-	extern int key_insert;
-	int		i, len;
-	char	c[256], *text;
+	extern	qpic_t *pic_ovr, *pic_ins; //johnfitz -- new cursor handling
+	extern	double key_blinktime;
+	extern	int key_insert;
+	int	i, len;
+	char	c[MAXCMDLINE], *text;
 
 	if (key_dest != key_console && !con_forcedup)
 		return;		// don't draw anything
@@ -1218,7 +1216,7 @@ void Con_DrawConsole (int lines, qboolean drawinput)
 Con_NotifyBox
 ==================
 */
-void Con_NotifyBox (char *text)
+void Con_NotifyBox (const char *text)
 {
 	double		t1, t2;
 
@@ -1247,3 +1245,38 @@ void Con_NotifyBox (char *text)
 	realtime = 0;				// put the cursor back to invisible
 }
 
+
+void LOG_Init (quakeparms_t *parms)
+{
+	time_t	inittime;
+	char	session[24];
+
+	if (!COM_CheckParm("-condebug"))
+		return;
+
+	inittime = time (NULL);
+	strftime (session, sizeof(session), "%m/%d/%Y %H:%M:%S", localtime(&inittime));
+	q_snprintf (logfilename, sizeof(logfilename), "%s/qconsole.log", parms->basedir);
+
+	unlink (logfilename);
+
+	log_fd = open (logfilename, O_WRONLY | O_CREAT | O_APPEND, 0666);
+	if (log_fd == -1)
+	{
+		fprintf (stderr, "Error: Unable to create log file %s\n", logfilename);
+		return;
+	}
+
+	con_debuglog = true;
+	Con_DebugLog (va("LOG started on: %s \n", session));
+
+}
+
+void LOG_Close (void)
+{
+	if (log_fd == -1)
+		return;
+	close (log_fd);
+	log_fd = -1;
+}
+
