@@ -212,13 +212,13 @@ void Scrap_Upload (void)
 Draw_PicFromWad
 ================
 */
-qpic_t *Draw_PicFromWad (char *name)
+qpic_t *Draw_PicFromWad (const char *name)
 {
 	qpic_t	*p;
 	glpic_t	gl;
 	src_offset_t offset; //johnfitz
 
-	p = W_GetLumpName (name);
+	p = (qpic_t *) W_GetLumpName (name);
 	if (!p) return pic_nul; //johnfitz
 
 	// load little ones into the scrap
@@ -268,7 +268,7 @@ qpic_t *Draw_PicFromWad (char *name)
 Draw_CachePic
 ================
 */
-qpic_t	*Draw_CachePic (char *path)
+qpic_t	*Draw_CachePic (const char *path)
 {
 	cachepic_t	*pic;
 	int			i;
@@ -371,13 +371,13 @@ qpic_t *Draw_ConbackPic (void)
 Draw_MakePic -- johnfitz -- generate pics from internal data
 ================
 */
-qpic_t *Draw_MakePic (char *name, int width, int height, byte *data)
+qpic_t *Draw_MakePic (const char *name, int width, int height, byte *data)
 {
 	int flags = TEXPREF_NEAREST | TEXPREF_ALPHA | TEXPREF_PERSIST | TEXPREF_NOPICMIP | TEXPREF_PAD;
 	qpic_t		*pic;
 	glpic_t		gl;
 
-	pic = Hunk_Alloc (sizeof(qpic_t) - 4 + sizeof (glpic_t));
+	pic = (qpic_t *) Hunk_Alloc (sizeof(qpic_t) - 4 + sizeof (glpic_t));
 	pic->width = width;
 	pic->height = height;
 
@@ -407,7 +407,7 @@ void Draw_LoadPics (void)
 	byte		*data;
 	src_offset_t	offset;
 
-	data = W_GetLumpName ("conchars");
+	data = (byte *) W_GetLumpName ("conchars");
 	if (!data) Sys_Error ("Draw_LoadPics: couldn't load conchars");
 	offset = (src_offset_t)data - (src_offset_t)wad_base;
 	char_texture = TexMgr_LoadImage (NULL, WADFILENAME":conchars", 128, 128, SRC_INDEXED, data,
@@ -531,7 +531,7 @@ void Draw_Character (int x, int y, int num)
 Draw_String -- johnfitz -- modified to call Draw_CharacterQuad
 ================
 */
-void Draw_String (int x, int y, char *str)
+void Draw_String (int x, int y, const char *str)
 {
 	if (y <= -8)
 		return;			// totally off screen
@@ -858,7 +858,7 @@ GL_Set2D -- johnfitz -- rewritten
 */
 void GL_Set2D (void)
 {
-	currentcanvas = -1;
+	currentcanvas = CANVAS_INVALID;
 	GL_SetCanvas (CANVAS_DEFAULT);
 
 	glDisable (GL_DEPTH_TEST);
