@@ -145,8 +145,8 @@ void Sky_LoadTexture (texture_t *mt)
 Sky_LoadSkyBox
 ==================
 */
-char	*suf[6] = {"rt", "bk", "lf", "ft", "up", "dn"};
-void Sky_LoadSkyBox (char *name)
+const char	*suf[6] = {"rt", "bk", "lf", "ft", "up", "dn"};
+void Sky_LoadSkyBox (const char *name)
 {
 	int		i, mark, width, height;
 	char	filename[MAX_OSPATH];
@@ -213,7 +213,7 @@ Sky_NewMap
 void Sky_NewMap (void)
 {
 	char	key[128], value[4096];
-	char	*data;
+	const char	*data;
 	int		i;
 
 	//
@@ -591,7 +591,7 @@ void Sky_ProcessEntities (void)
 				{
 					//copy the polygon and translate manually, since Sky_ProcessPoly needs it to be in world space
 					mark = Hunk_LowMark();
-					p = Hunk_Alloc (sizeof(*s->polys)); //FIXME: don't allocate for each poly
+					p = (glpoly_t *) Hunk_Alloc (sizeof(*s->polys)); //FIXME: don't allocate for each poly
 					p->numverts = s->polys->numverts;
 					for (k=0; k<p->numverts; k++)
 					{
@@ -891,7 +891,7 @@ void Sky_DrawFace (int axis)
 	Sky_SetBoxVert(1.0,  -1.0, axis, verts[3]);
 
 	start = Hunk_LowMark ();
-	p = Hunk_Alloc(sizeof(glpoly_t));
+	p = (glpoly_t *) Hunk_Alloc(sizeof(glpoly_t));
 
 	VectorSubtract(verts[2],verts[3],vup);
 	VectorSubtract(verts[2],verts[1],vright);
