@@ -34,8 +34,16 @@ const int	gl_alpha_format = 4;
 
 #define	MAX_GLTEXTURES	2048
 gltexture_t	*active_gltextures, *free_gltextures;
+gltexture_t	*notexture, *nulltexture;
 int numgltextures;
 
+unsigned int d_8to24table[256];
+unsigned int d_8to24table_fbright[256];
+unsigned int d_8to24table_nobright[256];
+unsigned int d_8to24table_conchars[256];
+unsigned int d_8to24table_shirt[256];
+unsigned int d_8to24table_pants[256];
+
 /*
 ================================================================================
 
@@ -47,8 +55,8 @@ int numgltextures;
 typedef struct
 {
 	int	magfilter;
-	int minfilter;
-	char *name;
+	int	minfilter;
+	const char  *name;
 } glmode_t;
 glmode_t modes[] = {
 	{GL_NEAREST, GL_NEAREST,				"GL_NEAREST"},
@@ -117,7 +125,7 @@ TexMgr_TextureMode_f
 void TexMgr_TextureMode_f (void)
 {
 	gltexture_t	*glt;
-	char *arg;
+	const char	*arg;
 	int i;
 
 	switch (Cmd_Argc())
@@ -237,13 +245,13 @@ void TexMgr_Imagedump_f (void)
 		GL_Bind (glt);
 		if (glt->flags & TEXPREF_ALPHA)
 		{
-			buffer = malloc(glt->width*glt->height*4);
+			buffer = (byte *) malloc(glt->width*glt->height*4);
 			glGetTexImage(GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, buffer);
 			Image_WriteTGA (tganame, buffer, glt->width, glt->height, 32, true);
 		}
 		else
 		{
-			buffer = malloc(glt->width*glt->height*3);
+			buffer = (byte *) malloc(glt->width*glt->height*3);
 			glGetTexImage(GL_TEXTURE_2D, 0, GL_RGB, GL_UNSIGNED_BYTE, buffer);
 			Image_WriteTGA (tganame, buffer, glt->width, glt->height, 24, true);
 		}
@@ -291,7 +299,7 @@ float TexMgr_FrameUsage (void)
 TexMgr_FindTexture
 ================
 */
-gltexture_t *TexMgr_FindTexture (model_t *owner, char *name)
+gltexture_t *TexMgr_FindTexture (model_t *owner, const char *name)
 {
 	gltexture_t	*glt;
 
@@ -430,7 +438,7 @@ void TexMgr_LoadPalette (void)
 		Sys_Error ("Couldn't load gfx/palette.lmp");
 
 	mark = Hunk_LowMark ();
-	pal = Hunk_Alloc (768);
+	pal = (byte *) Hunk_Alloc (768);
 	fread (pal, 1, 768, f);
 	fclose(f);
 
@@ -524,7 +532,7 @@ void TexMgr_RecalcWarpImageSize (void)
 	// resize the textures in opengl
 	//
 	mark = Hunk_LowMark();
-	dummy = Hunk_Alloc (gl_warpimagesize*gl_warpimagesize*4);
+	dummy = (byte *) Hunk_Alloc (gl_warpimagesize*gl_warpimagesize*4);
 
 	for (glt=active_gltextures; glt; glt=glt->next)
 	{
@@ -701,7 +709,7 @@ unsigned *TexMgr_ResampleTexture (unsigned *in, int inwidth, int inheight, qbool
 
 	outwidth = TexMgr_Pad(inwidth);
 	outheight = TexMgr_Pad(inheight);
-	out = Hunk_Alloc(outwidth*outheight*4);
+	out = (unsigned *) Hunk_Alloc(outwidth*outheight*4);
 
 	xfrac = ((inwidth-1) << 16) / (outwidth-1);
 	yfrac = ((inheight-1) << 16) / (outheight-1);
@@ -881,7 +889,7 @@ unsigned *TexMgr_8to32 (byte *in, int pixels, unsigned int *usepal)
 	int i;
 	unsigned *out, *data;
 
-	out = data = Hunk_Alloc(pixels*4);
+	out = data = (unsigned *) Hunk_Alloc(pixels*4);
 
 	for (i=0 ; i<pixels ; i++)
 		*out++ = usepal[*in++];
@@ -904,7 +912,7 @@ byte *TexMgr_PadImageW (byte *in, int width, int height, byte padbyte)
 
 	outwidth = TexMgr_Pad(width);
 
-	out = data = Hunk_Alloc(outwidth*height);
+	out = data = (byte *) Hunk_Alloc(outwidth*height);
 
 	for (i=0; i<height; i++)
 	{
@@ -933,7 +941,7 @@ byte *TexMgr_PadImageH (byte *in, int width, int height, byte padbyte)
 	srcpix = width * height;
 	dstpix = width * TexMgr_Pad(height);
 
-	out = data = Hunk_Alloc(dstpix);
+	out = data = (byte *) Hunk_Alloc(dstpix);
 
 	for (i=0; i<srcpix; i++)
 		*out++ = *in++;
@@ -1118,8 +1126,8 @@ void TexMgr_LoadLightmap (gltexture_t *glt, byte *data)
 TexMgr_LoadImage -- the one entry point for loading all textures
 ================
 */
-gltexture_t *TexMgr_LoadImage (model_t *owner, char *name, int width, int height, enum srcformat format,
-			       byte *data, char *source_file, src_offset_t source_offset, unsigned flags)
+gltexture_t *TexMgr_LoadImage (model_t *owner, const char *name, int width, int height, enum srcformat format,
+			       byte *data, const char *source_file, src_offset_t source_offset, unsigned flags)
 {
 	extern int lightmap_bytes;
 	unsigned short crc;
@@ -1271,7 +1279,7 @@ invalid:
 
 		//translate texture
 		size = glt->width * glt->height;
-		dst = translated = Hunk_Alloc (size);
+		dst = translated = (byte *) Hunk_Alloc (size);
 		src = data;
 
 		for (i=0; i<size; i++)
