@@ -119,7 +119,7 @@ void VID_MenuKey (int key);
 
 LONG WINAPI MainWndProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
 void AppActivate(BOOL fActive, BOOL minimize);
-char *VID_GetModeDescription (int mode);
+const char *VID_GetModeDescription (int mode);
 void ClearAllStates (void);
 void VID_UpdateWindowStatus (void);
 void GL_Init (void);
@@ -131,8 +131,8 @@ PROC glVertexPointerEXT;
 
 typedef void (APIENTRY *lp3DFXFUNC) (int, int, int, int, int, const void*);
 
-extern MTEXCOORDFUNC GL_MTexCoord2fFunc = NULL; //johnfitz
-extern SELECTTEXFUNC GL_SelectTextureFunc = NULL; //johnfitz
+PFNGLMULTITEXCOORD2FARBPROC GL_MTexCoord2fFunc = NULL; //johnfitz
+PFNGLACTIVETEXTUREARBPROC GL_SelectTextureFunc = NULL; //johnfitz
 
 typedef BOOL (APIENTRY * SETSWAPFUNC) (int); //johnfitz
 typedef int (APIENTRY * GETSWAPFUNC) (void); //johnfitz
@@ -251,7 +251,7 @@ void VID_Gamma_f (void)
 
 	oldgamma = vid_gamma.value;
 
- 	for (i=0; i<256; i++)
+	for (i=0; i<256; i++)
 		vid_gammaramp[i] = vid_gammaramp[i+256] = vid_gammaramp[i+512] =
 			CLAMP(0, (int) (255 * pow ((i+0.5)/255.5, vid_gamma.value) + 0.5), 255) << 8;
 
@@ -344,9 +344,9 @@ VID_SetWindowedMode
 */
 qboolean VID_SetWindowedMode (int modenum)
 {
-	HDC				hdc;
-	int				lastmodestate, width, height;
-	RECT			rect;
+	HDC		hdc;
+	int		lastmodestate, width, height;
+	RECT		rect;
 
 	lastmodestate = modestate;
 
@@ -430,9 +430,9 @@ VID_SetFullDIBMode
 */
 qboolean VID_SetFullDIBMode (int modenum)
 {
-	HDC				hdc;
-	int				lastmodestate, width, height;
-	RECT			rect;
+	HDC		hdc;
+	int		lastmodestate, width, height;
+	RECT		rect;
 
 	if (!leavecurrentmode)
 	{
@@ -526,10 +526,10 @@ VID_SetMode
 */
 int VID_SetMode (int modenum)
 {
-	int				original_mode, temp;
-	qboolean		stat;
-    MSG				msg;
-	HDC				hdc;
+	int		original_mode, temp;
+	qboolean	stat = false;
+	MSG		msg;
+	HDC		hdc;
 
 	if ((windowed && (modenum != 0)) ||
 		(!windowed && (modenum < 1)) ||
@@ -576,6 +576,11 @@ int VID_SetMode (int modenum)
 		Sys_Error ("VID_SetMode: Bad mode type in modelist");
 	}
 
+	if (!stat)
+	{
+		Sys_Error ("Couldn't set video mode");
+	}
+
 	//johnfitz -- re-initialize dinput becuase it's tied to the "mainwindow" object
 	if (COM_CheckParm ("-dinput"))
 		IN_InitDInput ();
@@ -588,11 +593,6 @@ int VID_SetMode (int modenum)
 	CDAudio_Resume ();
 	scr_disabled_for_loading = temp;
 
-	if (!stat)
-	{
-		Sys_Error ("Couldn't set video mode");
-	}
-
 // now we try to make sure we get the focus on the mode switch, because
 // sometimes in some systems we don't.  We grab the foreground, then
 // finish setting up, pump all our messages, and sleep for a little while
@@ -604,8 +604,8 @@ int VID_SetMode (int modenum)
 
 	while (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE))
 	{
-      	TranslateMessage (&msg);
-      	DispatchMessage (&msg);
+		TranslateMessage (&msg);
+		DispatchMessage (&msg);
 	}
 
 	Sleep (100);
@@ -656,9 +656,9 @@ VID_Restart -- johnfitz -- change video modes on the fly
 */
 void VID_Restart (void)
 {
-	HDC			hdc;
+	HDC		hdc;
 	HGLRC		hrc;
-	int			i;
+	int		i;
 	qboolean	mode_changed = false;
 	vmode_t		oldmode;
 
@@ -930,8 +930,7 @@ char *GL_MakeNiceExtensionsList (const char *in)
 	out = Z_Malloc (strlen(in) + count*3 + 1); //usually about 1-2k
 	out[0] = 0;
 
-	copy = Z_Malloc(strlen(in) + 1);
-	strcpy(copy, in);
+	copy = (char *) Z_Strdup(in);
 
 	for (token = strtok(copy, " "); token; token = strtok(NULL, " "))
 	{
@@ -971,24 +970,22 @@ void GL_Info_f (void)
 CheckArrayExtensions
 ===============
 */
+#if 0 /* unused */
 void CheckArrayExtensions (void)
 {
-	char		*tmp;
+	const char	*tmp;
 
-	/* check for texture extension */
-	tmp = (unsigned char *)glGetString(GL_EXTENSIONS);
+	tmp = (const char *)glGetString(GL_EXTENSIONS);
 	while (*tmp)
 	{
-		if (strncmp((const char*)tmp, "GL_EXT_vertex_array", strlen("GL_EXT_vertex_array")) == 0)
+		if (strncmp(tmp, "GL_EXT_vertex_array", sizeof("GL_EXT_vertex_array") -1) == 0)
 		{
-			if (
-((glArrayElementEXT = wglGetProcAddress("glArrayElementEXT")) == NULL) ||
-((glColorPointerEXT = wglGetProcAddress("glColorPointerEXT")) == NULL) ||
-((glTexCoordPointerEXT = wglGetProcAddress("glTexCoordPointerEXT")) == NULL) ||
-((glVertexPointerEXT = wglGetProcAddress("glVertexPointerEXT")) == NULL) )
+			if (((glArrayElementEXT = wglGetProcAddress("glArrayElementEXT")) == NULL) ||
+			    ((glColorPointerEXT = wglGetProcAddress("glColorPointerEXT")) == NULL) ||
+			    ((glTexCoordPointerEXT = wglGetProcAddress("glTexCoordPointerEXT")) == NULL) ||
+			    ((glVertexPointerEXT = wglGetProcAddress("glVertexPointerEXT")) == NULL) )
 			{
 				Sys_Error ("GetProcAddress for vertex extension failed");
-				return;
 			}
 			return;
 		}
@@ -997,6 +994,7 @@ void CheckArrayExtensions (void)
 
 	Sys_Error ("Vertex array extension not present");
 }
+#endif /* #if 0 */
 
 /*
 ===============
@@ -1114,7 +1112,7 @@ void GL_CheckExtensions (void)
 	if (strstr(gl_extensions, "GL_EXT_texture_filter_anisotropic"))
 	{
 		float test1,test2;
-		int tex;
+		GLuint tex;
 
 		// test to make sure we really have control over it
 		// 1.0 and 2.0 should always be legal values
@@ -1195,10 +1193,10 @@ GL_Init
 */
 void GL_Init (void)
 {
-	gl_vendor = glGetString (GL_VENDOR);
-	gl_renderer = glGetString (GL_RENDERER);
-	gl_version = glGetString (GL_VERSION);
-	gl_extensions = glGetString (GL_EXTENSIONS);
+	gl_vendor = (const char *) glGetString (GL_VENDOR);
+	gl_renderer = (const char *) glGetString (GL_RENDERER);
+	gl_version = (const char *) glGetString (GL_VERSION);
+	gl_extensions = (const char *) glGetString (GL_EXTENSIONS);
 
 	GetWGLExtensions (); //johnfitz
 	GL_CheckExtensions (); //johnfitz
@@ -1207,11 +1205,11 @@ void GL_Init (void)
 
 	Cvar_RegisterVariable (&vid_vsync, VID_Vsync_f); //johnfitz
 
-    if (strnicmp(gl_renderer,"PowerVR",7)==0)
-         fullsbardraw = true;
+	if (strnicmp(gl_renderer,"PowerVR",7)==0)
+		fullsbardraw = true;
 
-    if (strnicmp(gl_renderer,"Permedia",8)==0)
-         isPermedia = true;
+	if (strnicmp(gl_renderer,"Permedia",8)==0)
+		isPermedia = true;
 
 	//johnfitz -- intel video workarounds from Baker
 	if (!strcmp(gl_vendor, "Intel"))
@@ -1287,19 +1285,19 @@ void VID_SetDefaultMode (void)
 
 void	VID_Shutdown (void)
 {
-   	HGLRC hRC;
-   	HDC	  hDC;
+	HGLRC	hRC;
+	HDC	hDC;
 
 	if (vid_initialized)
 	{
 		vid_canalttab = false;
 		hRC = wglGetCurrentContext();
-    	hDC = wglGetCurrentDC();
+		hDC = wglGetCurrentDC();
 
-    	wglMakeCurrent(NULL, NULL);
+		wglMakeCurrent(NULL, NULL);
 
-    	if (hRC)
-    	    wglDeleteContext(hRC);
+		if (hRC)
+			wglDeleteContext(hRC);
 
 		VID_Gamma_Shutdown (); //johnfitz
 
@@ -1572,15 +1570,15 @@ LONG WINAPI MainWndProc (
     WPARAM  wParam,
     LPARAM  lParam)
 {
-    LONG    lRet = 1;
+	LONG    lRet = 1;
 	int		fwKeys, xPos, yPos, fActive, fMinimized, temp;
 	extern unsigned int uiWheelMessage;
 
 	if ( uMsg == uiWheelMessage )
 		uMsg = WM_MOUSEWHEEL;
 
-    switch (uMsg)
-    {
+	switch (uMsg)
+	{
 		case WM_KILLFOCUS:
 			if (modestate == MS_FULLDIB)
 				ShowWindow(mainwindow, SW_SHOWMINNOACTIVE);
@@ -1646,17 +1644,17 @@ LONG WINAPI MainWndProc (
 			}
 			break;
 
-    	case WM_SIZE:
-            break;
+		case WM_SIZE:
+			break;
 
-   	    case WM_CLOSE:
+		case WM_CLOSE:
 			if (MessageBox (mainwindow, "Are you sure you want to quit?", "Confirm Exit",
 						MB_YESNO | MB_SETFOREGROUND | MB_ICONQUESTION) == IDYES)
 			{
 				Sys_Quit ();
 			}
 
-	        break;
+			break;
 
 		case WM_ACTIVATE:
 			fActive = LOWORD(wParam);
@@ -1668,27 +1666,27 @@ LONG WINAPI MainWndProc (
 
 			break;
 
-   	    case WM_DESTROY:
-        {
+		case WM_DESTROY:
+			{
 			if (dibwindow)
 				DestroyWindow (dibwindow);
 
-            PostQuitMessage (0);
-        }
-        break;
-
-		case MM_MCINOTIFY:
-            lRet = CDAudio_MessageHandler (hWnd, uMsg, wParam, lParam);
+			PostQuitMessage (0);
+			}
 			break;
 
-    	default:
-            /* pass all unhandled messages to DefWindowProc */
-            lRet = DefWindowProc (hWnd, uMsg, wParam, lParam);
-        break;
-    }
+		case MM_MCINOTIFY:
+			lRet = CDAudio_MessageHandler (hWnd, uMsg, wParam, lParam);
+			break;
 
-    /* return 1 if handled message, 0 if not */
-    return lRet;
+		default:
+			/* pass all unhandled messages to DefWindowProc */
+			lRet = DefWindowProc (hWnd, uMsg, wParam, lParam);
+			break;
+	}
+
+	/* return 1 if handled message, 0 if not */
+	return lRet;
 }
 
 //==========================================================================
@@ -1728,9 +1726,9 @@ vmode_t *VID_GetModePtr (int modenum)
 VID_GetModeDescription
 =================
 */
-char *VID_GetModeDescription (int mode)
+const char *VID_GetModeDescription (int mode)
 {
-	char		*pinfo;
+	const char	*pinfo;
 	vmode_t		*pv;
 	static char	temp[100];
 
@@ -1760,7 +1758,7 @@ char *VID_GetModeDescription (int mode)
 VID_GetExtModeDescription
 =================
 */
-char *VID_GetExtModeDescription (int mode)
+const char *VID_GetExtModeDescription (int mode)
 {
 	static char	pinfo[40];
 	vmode_t		*pv;
@@ -1811,10 +1809,9 @@ VID_DescribeModes_f -- johnfitz -- changed formatting, and added refresh rates a
 */
 void VID_DescribeModes_f (void)
 {
-	int			i, lnummodes, t;
-	char		*pinfo;
+	int		i, lnummodes, t;
 	vmode_t		*pv;
-	int			lastwidth=0, lastheight=0, lastbpp=0, count=0;
+	int		lastwidth=0, lastheight=0, lastbpp=0, count=0;
 
 	lnummodes = VID_NumModes ();
 
@@ -1863,18 +1860,18 @@ void VID_InitDIB (HINSTANCE hInstance)
 	int				i;
 
 	/* Register the frame class */
-    wc.style         = 0;
-    wc.lpfnWndProc   = (WNDPROC)MainWndProc;
-    wc.cbClsExtra    = 0;
-    wc.cbWndExtra    = 0;
-    wc.hInstance     = hInstance;
-    wc.hIcon         = 0;
-    wc.hCursor       = LoadCursor (NULL,IDC_ARROW);
+	wc.style         = 0;
+	wc.lpfnWndProc   = (WNDPROC)MainWndProc;
+	wc.cbClsExtra    = 0;
+	wc.cbWndExtra    = 0;
+	wc.hInstance     = hInstance;
+	wc.hIcon         = 0;
+	wc.hCursor       = LoadCursor (NULL,IDC_ARROW);
 	wc.hbrBackground = NULL;
-    wc.lpszMenuName  = 0;
-    wc.lpszClassName = "FitzQuake"; //johnfitz -- was WinQuake
+	wc.lpszMenuName  = 0;
+	wc.lpszClassName = "FitzQuake"; //johnfitz -- was WinQuake
 
-    if (!RegisterClass (&wc) )
+	if (!RegisterClass (&wc) )
 		Sys_Error ("Couldn't register window class");
 
 	modelist[0].type = MS_WINDOWED;
@@ -2182,6 +2179,8 @@ void	VID_Init (void)
 
 				if (COM_CheckParm("-height"))
 					height = Q_atoi(com_argv[COM_CheckParm("-height")+1]);
+				else
+					height = width * 3 / 4; // assume 4:3 aspect ratio
 
 			// if they want to force it, add the specified mode to the list
 				if (COM_CheckParm("-force") && (nummodes < MAX_MODE_LIST))
@@ -2297,18 +2296,18 @@ void	VID_Init (void)
 
 	VID_SetMode (vid_default);
 
-    maindc = GetDC(mainwindow);
+	maindc = GetDC(mainwindow);
 	bSetupPixelFormat(maindc);
 
-    baseRC = wglCreateContext( maindc );
+	baseRC = wglCreateContext( maindc );
 	if (!baseRC)
 		Sys_Error ("Could not initialize GL (wglCreateContext failed).\n\nMake sure you in are 65535 color mode, and try running -window.");
-    if (!wglMakeCurrent( maindc, baseRC ))
+	if (!wglMakeCurrent( maindc, baseRC ))
 		Sys_Error ("VID_Init: wglMakeCurrent failed");
 
 	GL_Init ();
 
-	//johnfitz -- removed code to create "glquake" subdirectory
+	//johnfitz -- removed code creating "glquake" subdirectory
 
 	vid_realmode = vid_modenum;
 
@@ -2355,38 +2354,6 @@ void VID_SyncCvars (void)
 //
 //==========================================================================
 
-extern void M_Menu_Options_f (void);
-extern void M_Print (int cx, int cy, char *str);
-extern void M_PrintWhite (int cx, int cy, char *str);
-extern void M_DrawCharacter (int cx, int line, int num);
-extern void M_DrawTransPic (int x, int y, qpic_t *pic);
-extern void M_DrawPic (int x, int y, qpic_t *pic);
-extern void M_DrawCheckbox (int x, int y, int on);
-
-extern qboolean	m_entersound;
-
-enum {
-	m_none,
-	m_main,
-	m_singleplayer,
-	m_load,
-	m_save,
-	m_multiplayer,
-	m_setup,
-	m_net,
-	m_options,
-	m_video,
-	m_keys,
-	m_help,
-	m_quit,
-	m_serialconfig,
-	m_modemconfig,
-	m_lanconfig,
-	m_gameoptions,
-	m_search,
-	m_slist
-} m_state;
-
 #define VIDEO_OPTIONS_ITEMS 6
 int		video_cursor_table[] = {48, 56, 64, 72, 88, 96};
 int		video_options_cursor = 0;
@@ -2708,13 +2675,13 @@ void VID_MenuKey (int key)
 		switch (video_options_cursor)
 		{
 		case 0:
-			VID_Menu_ChooseNextMode (-1);
+			VID_Menu_ChooseNextMode (1);
 			break;
 		case 1:
-			VID_Menu_ChooseNextBpp (-1);
+			VID_Menu_ChooseNextBpp (1);
 			break;
 		case 2:
-			VID_Menu_ChooseNextRate (-1);
+			VID_Menu_ChooseNextRate (1);
 			break;
 		case 3:
 			Cbuf_AddText ("toggle vid_fullscreen\n");
@@ -2731,13 +2698,13 @@ void VID_MenuKey (int key)
 		switch (video_options_cursor)
 		{
 		case 0:
-			VID_Menu_ChooseNextMode (1);
+			VID_Menu_ChooseNextMode (-1);
 			break;
 		case 1:
-			VID_Menu_ChooseNextBpp (1);
+			VID_Menu_ChooseNextBpp (-1);
 			break;
 		case 2:
-			VID_Menu_ChooseNextRate (1);
+			VID_Menu_ChooseNextRate (-1);
 			break;
 		case 3:
 			Cbuf_AddText ("toggle vid_fullscreen\n");
@@ -2790,7 +2757,7 @@ void VID_MenuDraw (void)
 {
 	int i = 0;
 	qpic_t *p;
-	char *title;
+	const char *title;
 
 	M_DrawTransPic (16, 4, Draw_CachePic ("gfx/qplaque.lmp"));
 
