@@ -1,7 +1,8 @@
 /*
 Copyright (C) 1996-2001 Id Software, Inc.
-Copyright (C) 2002-2005 John Fitzgibbons and others
+Copyright (C) 2002-2009 John Fitzgibbons and others
 Copyright (C) 2007-2008 Kristian Duske
+Copyright (C) 2010 Ozkan Sezer and Steven Atkinson
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,6 +24,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "quakedef.h"
 #include "resource.h"
+#include "SDL.h"
 
 #define MAX_MODE_LIST	600 //johnfitz -- was 30
 #define VID_ROW_SIZE	3
@@ -43,6 +45,7 @@ typedef struct {
 	int			dib;
 	int			fullscreen;
 	int			bpp;
+//	int			refreshrate; //johnfitz
 	int			halfscreen;
 	char		modedesc[17];
 } vmode_t;
@@ -91,12 +94,6 @@ viddef_t	vid;				// global video state
 //unsigned short	d_8to16table[256]; //johnfitz -- never used
 //unsigned char		d_15to8table[65536]; //johnfitz -- never used
 
-
-PFNGLARRAYELEMENTEXTPROC glArrayElementEXT = NULL;
-PFNGLCOLORPOINTEREXTPROC glColorPointerEXT = NULL;
-PFNGLTEXCOORDPOINTEREXTPROC glTexCoordPointerEXT = NULL;
-PFNGLVERTEXPOINTEREXTPROC glVertexPointerEXT = NULL;
-
 modestate_t	modestate = MODE_WINDOWED;
 
 void VID_Menu_Init (void); //johnfitz
@@ -104,11 +101,16 @@ void VID_Menu_f (void); //johnfitz
 void VID_MenuDraw (void);
 void VID_MenuKey (int key);
 
-char *VID_GetModeDescription (int mode);
+const char *VID_GetModeDescription (int mode);
 void ClearAllStates (void);
 void VID_UpdateWindowStatus (void);
 void GL_Init (void);
 
+PFNGLARRAYELEMENTEXTPROC glArrayElementEXT = NULL;
+PFNGLCOLORPOINTEREXTPROC glColorPointerEXT = NULL;
+PFNGLTEXCOORDPOINTEREXTPROC glTexCoordPointerEXT = NULL;
+PFNGLVERTEXPOINTEREXTPROC glVertexPointerEXT = NULL;
+
 PFNGLMULTITEXCOORD2FARBPROC GL_MTexCoord2fFunc = NULL; //johnfitz
 PFNGLACTIVETEXTUREARBPROC GL_SelectTextureFunc = NULL; //johnfitz
 
@@ -131,9 +133,9 @@ void GL_SetupState (void); //johnfitz
 //====================================
 
 //johnfitz -- new cvars
-cvar_t		vid_fullscreen = {"vid_fullscreen", "0", true};
-cvar_t		vid_width = {"vid_width", "800", true};
-cvar_t		vid_height = {"vid_height", "600", true};
+cvar_t		vid_fullscreen = {"vid_fullscreen", "0", true};	// QuakeSpasm, was "1"
+cvar_t		vid_width = {"vid_width", "800", true};		// QuakeSpasm, was 640
+cvar_t		vid_height = {"vid_height", "600", true};	// QuakeSpasm, was 480
 cvar_t		vid_bpp = {"vid_bpp", "16", true};
 cvar_t		vid_refreshrate = {"vid_refreshrate", "60", true};
 cvar_t		vid_vsync = {"vid_vsync", "0", true};
@@ -209,7 +211,8 @@ void VID_Gamma_f (void)
 
 	for (i=0; i<256; i++)
 	{
-		vid_gamma_red[i] = CLAMP(0, (int) (255 * pow ((i+0.5)/255.5, vid_gamma.value) + 0.5), 255) << 8;
+		vid_gamma_red[i] =
+			CLAMP(0, (int) (255 * pow ((i+0.5)/255.5, vid_gamma.value) + 0.5), 255) << 8;
 		vid_gamma_green[i] = vid_gamma_red[i];
 		vid_gamma_blue[i] = vid_gamma_red[i];
 	}
@@ -239,10 +242,10 @@ VID_SetMode
 */
 int VID_SetMode (int modenum)
 {
-	int temp;
-	qboolean stat;
-	Uint32 flags = SDL_DEFAULT_FLAGS;
-	char caption[50];
+	int		temp;
+	qboolean	stat = false;
+	Uint32	flags = SDL_DEFAULT_FLAGS;
+	char		caption[50];
 
 // TODO: check if video mode is supported using SDL_VideoModeOk
 	if ((windowed && (modenum != 0)) ||
@@ -306,9 +309,14 @@ int VID_SetMode (int modenum)
 		Sys_Error ("VID_SetMode: Bad mode type in modelist");
 	}
 
+	if (!stat)
+	{
+		Sys_Error ("Couldn't set video mode");
+	}
+
 	//kristian -- set window caption
-	sprintf(caption, "FitzQuake (SDL port) Version %1.2f", FITZQUAKE_VERSION);
-	SDL_WM_SetCaption((const char* )&caption, (const char*)&caption);
+	sprintf(caption, "QuakeSpasm %1.2f.%d", (float)FITZQUAKE_VERSION, QUAKESPASM_VER_PATCH);
+	SDL_WM_SetCaption(caption, caption);
 
 	vid.width = modelist[modenum].width;
 	vid.height = modelist[modenum].height;
@@ -322,11 +330,6 @@ int VID_SetMode (int modenum)
 	CDAudio_Resume ();
 	scr_disabled_for_loading = temp;
 
-	if (!stat)
-	{
-		Sys_Error ("Couldn't set video mode");
-	}
-
 	vid_modenum = modenum;
 
 // fix the leftover Alt from any Alt-Tab or the like that switched us away
@@ -364,7 +367,7 @@ VID_Restart -- johnfitz -- change video modes on the fly
 */
 void VID_Restart (void)
 {
-	int			i;
+	int		i;
 	vmode_t		oldmode;
 
 	if (vid_locked || !vid_changed)
@@ -435,7 +438,7 @@ void VID_Restart (void)
 	TexMgr_RecalcWarpImageSize ();
 
 	//conwidth and conheight need to be recalculated
-	vid.conwidth = (scr_conwidth.value > 0) ? (int)scr_conwidth.value : vid.width;
+	vid.conwidth = (scr_conwidth.value > 0) ? (int)scr_conwidth.value : (scr_conscale.value > 0) ? (int)(vid.width/scr_conscale.value) : vid.width;
 	vid.conwidth = CLAMP (320, vid.conwidth, vid.width);
 	vid.conwidth &= 0xFFFFFFF8;
 	vid.conheight = vid.conwidth * vid.height / vid.width;
@@ -524,11 +527,10 @@ char *GL_MakeNiceExtensionsList (const char *in)
 	for (i = 0, count = 1; i < strlen(in); i++)
 		if (in[i] == ' ')
 			count++;
-	out = Z_Malloc (strlen(in) + count*3 + 1); //usually about 1-2k
+	out = (char *) Z_Malloc (strlen(in) + count*3 + 1); //usually about 1-2k
 	out[0] = 0;
 
-	copy = Z_Malloc(strlen(in) + 1);
-	strcpy(copy, in);
+	copy = (char *) Z_Strdup(in);
 
 	for (token = strtok(copy, " "); token; token = strtok(NULL, " "))
 	{
@@ -563,15 +565,15 @@ void GL_Info_f (void)
 CheckArrayExtensions
 ===============
 */
+#if 0 /* unused */
 void CheckArrayExtensions (void)
 {
-	unsigned char *tmp;
+	const char	*tmp;
 
-	/* check for texture extension */
-	tmp = (GLubyte *)glGetString(GL_EXTENSIONS);
+	tmp = (const char *)glGetString(GL_EXTENSIONS);
 	while (*tmp)
 	{
-		if (strncmp((const char*)tmp, "GL_EXT_vertex_array", strlen("GL_EXT_vertex_array")) == 0)
+		if (strncmp(tmp, "GL_EXT_vertex_array", sizeof("GL_EXT_vertex_array") -1) == 0)
 		{
 			if (((glArrayElementEXT = SDL_GL_GetProcAddress("glArrayElementEXT")) == NULL) ||
 			    ((glColorPointerEXT = SDL_GL_GetProcAddress("glColorPointerEXT")) == NULL) ||
@@ -579,7 +581,6 @@ void CheckArrayExtensions (void)
 			    ((glVertexPointerEXT = SDL_GL_GetProcAddress("glVertexPointerEXT")) == NULL) )
 			{
 				Sys_Error ("GetProcAddress for vertex extension failed");
-				return;
 			}
 			return;
 		}
@@ -588,6 +589,7 @@ void CheckArrayExtensions (void)
 
 	Sys_Error ("Vertex array extension not present");
 }
+#endif /* #if 0 */
 
 /*
 ===============
@@ -602,12 +604,14 @@ void GL_CheckExtensions (void)
 	// multitexture
 	//
 	if (COM_CheckParm("-nomtex"))
-		Con_Printf ("WARNING: Mutitexture disabled at command line\n");
+		Con_Warning ("Mutitexture disabled at command line\n");
 	else
 		if (strstr(gl_extensions, "GL_ARB_multitexture"))
 		{
-			GL_MTexCoord2fFunc = SDL_GL_GetProcAddress("glMultiTexCoord2fARB");
-			GL_SelectTextureFunc = SDL_GL_GetProcAddress("glActiveTextureARB");
+			GL_MTexCoord2fFunc = (PFNGLMULTITEXCOORD2FARBPROC)
+						SDL_GL_GetProcAddress("glMultiTexCoord2fARB");
+			GL_SelectTextureFunc = (PFNGLACTIVETEXTUREARBPROC)
+						SDL_GL_GetProcAddress("glActiveTextureARB");
 			if (GL_MTexCoord2fFunc && GL_SelectTextureFunc)
 			{
 				Con_Printf("FOUND: ARB_multitexture\n");
@@ -616,13 +620,15 @@ void GL_CheckExtensions (void)
 				gl_mtexable = true;
 			}
 			else
-				Con_Printf ("WARNING: multitexture not supported (SDL_GL_GetProcAddress failed)\n");
+				Con_Warning ("multitexture not supported (SDL_GL_GetProcAddress failed)\n");
 		}
 		else
 			if (strstr(gl_extensions, "GL_SGIS_multitexture"))
 			{
-				GL_MTexCoord2fFunc = SDL_GL_GetProcAddress("glMTexCoord2fSGIS");
-				GL_SelectTextureFunc = SDL_GL_GetProcAddress("glSelectTextureSGIS");
+				GL_MTexCoord2fFunc = (PFNGLMULTITEXCOORD2FARBPROC)
+							SDL_GL_GetProcAddress("glMTexCoord2fSGIS");
+				GL_SelectTextureFunc = (PFNGLACTIVETEXTUREARBPROC)
+							SDL_GL_GetProcAddress("glSelectTextureSGIS");
 				if (GL_MTexCoord2fFunc && GL_SelectTextureFunc)
 				{
 					Con_Printf("FOUND: SGIS_multitexture\n");
@@ -631,16 +637,16 @@ void GL_CheckExtensions (void)
 					gl_mtexable = true;
 				}
 				else
-					Con_Printf ("WARNING: multitexture not supported (SDL_GL_GetProcAddress failed)\n");
+					Con_Warning ("multitexture not supported (SDL_GL_GetProcAddress failed)\n");
 
 			}
 			else
-				Con_Printf ("WARNING: multitexture not supported (extension not found)\n");
+				Con_Warning ("multitexture not supported (extension not found)\n");
 	//
 	// texture_env_combine
 	//
 	if (COM_CheckParm("-nocombine"))
-		Con_Printf ("WARNING: texture_env_combine disabled at command line\n");
+		Con_Warning ("texture_env_combine disabled at command line\n");
 	else
 		if (strstr(gl_extensions, "GL_ARB_texture_env_combine"))
 		{
@@ -654,9 +660,7 @@ void GL_CheckExtensions (void)
 				gl_texture_env_combine = true;
 			}
 			else
-				Con_Printf ("WARNING: texture_env_combine not supported\n");
-
-#if 1
+				Con_Warning ("texture_env_combine not supported\n");
 	//
 	// texture_env_add
 	//
@@ -676,7 +680,6 @@ void GL_CheckExtensions (void)
 			}
 			else
 				Con_Warning ("texture_env_add not supported\n");
-#endif
 
 	//
 	// swap control
@@ -685,18 +688,18 @@ void GL_CheckExtensions (void)
 	{
 		if (SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 0) == -1)
 		{
-			Con_Printf("WARNING: vertical sync not supported (SDL_GL_SetAttribute failed)\n");
+			Con_Warning ("vertical sync not supported (SDL_GL_SetAttribute failed)\n");
 		}
 		else
 		{
 			if (SDL_GL_GetAttribute(SDL_GL_SWAP_CONTROL, &swap_control) == -1)
 			{
-				Con_Printf("WARNING: vertical sync not supported (SDL_GL_GetAttribute failed). Make sure you don't have vertical sync disabled in your driver settings.\n");
+				Con_Warning ("vertical sync not supported (SDL_GL_GetAttribute failed). Make sure you don't have vertical sync disabled in your driver settings.\n");
 			}
 			else if (swap_control == -1)
 			{
 			// TODO: check if this is correct - I don't know what SDL returns if vertical sync is disabled
-				Con_Printf("WARNING: vertical sync not supported (swap interval is -1.) Make sure you don't have vertical sync disabled in your driver settings.\n");
+				Con_Warning ("vertical sync not supported (swap interval is -1.) Make sure you don't have vertical sync disabled in your driver settings.\n");
 			}
 			else
 			{
@@ -706,7 +709,7 @@ void GL_CheckExtensions (void)
 		}
 	}
 	else
-		Con_Printf ("WARNING: vertical sync not supported (extension not found)\n");
+		Con_Warning ("vertical sync not supported (extension not found)\n");
 
 	//
 	// anisotropic filtering
@@ -732,13 +735,13 @@ void GL_CheckExtensions (void)
 			gl_anisotropy_able = true;
 		}
 		else
-			Con_Printf("WARNING: anisotropic filtering locked by driver. Current driver setting is %f\n", test1);
+			Con_Warning ("anisotropic filtering locked by driver. Current driver setting is %f\n", test1);
 
 		//get max value either way, so the menu and stuff know it
 		glGetFloatv (GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &gl_max_anisotropy);
 	}
 	else
-		Con_Printf ("WARNING: texture_filter_anisotropic not supported\n");
+		Con_Warning ("texture_filter_anisotropic not supported\n");
 }
 
 /*
@@ -792,7 +795,6 @@ void GL_Init (void)
 		fullsbardraw = true;
 	if (SDL_strncasecmp(gl_renderer,"Permedia",8)==0)
 		isPermedia = true;
-#if 1
 	//johnfitz -- intel video workarounds from Baker
 	if (!strcmp(gl_vendor, "Intel"))
 	{
@@ -800,13 +802,12 @@ void GL_Init (void)
 		isIntelVideo = true;
 	}
 	//johnfitz
-#endif
 
 #if 0
 	//johnfitz -- confirm presence of stencil buffer
 	glGetIntegerv(GL_STENCIL_BITS, &gl_stencilbits);
 	if(!gl_stencilbits)
-		Con_Printf ("WARNING: Could not create stencil buffer\n");
+		Con_Warning ("Could not create stencil buffer\n");
 	else
 		Con_Printf ("%i bit stencil buffer\n", gl_stencilbits);
 #endif
@@ -927,9 +928,9 @@ vmode_t *VID_GetModePtr (int modenum)
 VID_GetModeDescription
 =================
 */
-char *VID_GetModeDescription (int mode)
+const char *VID_GetModeDescription (int mode)
 {
-	char		*pinfo;
+	const char	*pinfo;
 	vmode_t		*pv;
 	static char	temp[100];
 
@@ -959,7 +960,7 @@ char *VID_GetModeDescription (int mode)
 VID_GetExtModeDescription
 =================
 */
-char *VID_GetExtModeDescription (int mode)
+const char *VID_GetExtModeDescription (int mode)
 {
 	static char	pinfo[40];
 	vmode_t		*pv;
@@ -1058,7 +1059,7 @@ void VID_InitDIB (void)
 	if (COM_CheckParm("-width"))
 		modelist[0].width = Q_atoi(com_argv[COM_CheckParm("-width")+1]);
 	else
-		modelist[0].width = 800;
+		modelist[0].width = 800;	// QuakeSpasm, was 640
 
 	if (modelist[0].width < 320)
 		modelist[0].width = 320;
@@ -1181,10 +1182,10 @@ VID_Init
 */
 void	VID_Init (void)
 {
+	static char vid_center[] = "SDL_VIDEO_CENTERED=center";
 	const SDL_VideoInfo *info;
 	int		i, existingmode;
 	int		basenummodes, width, height, bpp, findbpp, done;
-	char		gldir[MAX_OSPATH];
 
 	//johnfitz -- clean up init readouts
 	//Con_Printf("------------- Init Video -------------\n");
@@ -1207,7 +1208,7 @@ void	VID_Init (void)
 	if (SDL_InitSubSystem(SDL_INIT_VIDEO) == -1)
 		Sys_Error("Could not initialize SDL Video");
 
-	SDL_putenv("SDL_VIDEO_CENTERED=center");
+	SDL_putenv (vid_center);
 
 	VID_InitDIB();
 	basenummodes = nummodes = 1;
@@ -1268,6 +1269,8 @@ void	VID_Init (void)
 
 				if (COM_CheckParm("-height"))
 					height = Q_atoi(com_argv[COM_CheckParm("-height")+1]);
+				else
+					height = width * 3 / 4; // assume 4:3 aspect ratio
 
 			// if they want to force it, add the specified mode to the list
 				if (COM_CheckParm("-force") && (nummodes < MAX_MODE_LIST))
@@ -1381,10 +1384,10 @@ void	VID_Init (void)
 	PL_SetWindowIcon();
 
 	VID_SetMode (vid_default);
+
 	GL_Init ();
 
-	sprintf (gldir, "%s/glquake", com_gamedir);
-	Sys_mkdir (gldir);
+	//johnfitz -- removed code creating "glquake" subdirectory
 
 	vid_realmode = vid_modenum;
 
@@ -1428,6 +1431,11 @@ void	VID_Toggle (void)
 	{
 		Sbar_Changed ();	// Sbar seems to need refreshing
 		windowed = !windowed;
+		if (vid.type == MODE_FULLSCREEN_DEFAULT)
+		    vid.type = MODE_WINDOWED;
+		else
+		    vid.type = MODE_FULLSCREEN_DEFAULT;
+
 		Cvar_SetValue ("vid_fullscreen", ! (int)vid_fullscreen.value);
 	}
 	else
@@ -1440,7 +1448,6 @@ vrestart:
 	}
 }
 
-
 /*
 ================
 VID_SyncCvars -- johnfitz -- set vid cvars to match current video mode
@@ -1465,38 +1472,6 @@ void VID_SyncCvars (void)
 //
 //==========================================================================
 
-extern void M_Menu_Options_f (void);
-extern void M_Print (int cx, int cy, char *str);
-extern void M_PrintWhite (int cx, int cy, char *str);
-extern void M_DrawCharacter (int cx, int line, int num);
-extern void M_DrawTransPic (int x, int y, qpic_t *pic);
-extern void M_DrawPic (int x, int y, qpic_t *pic);
-extern void M_DrawCheckbox (int x, int y, int on);
-
-extern qboolean	m_entersound;
-
-enum {
-	m_none,
-	m_main,
-	m_singleplayer,
-	m_load,
-	m_save,
-	m_multiplayer,
-	m_setup,
-	m_net,
-	m_options,
-	m_video,
-	m_keys,
-	m_help,
-	m_quit,
-	m_serialconfig,
-	m_modemconfig,
-	m_lanconfig,
-	m_gameoptions,
-	m_search,
-	m_slist
-} m_state;
-
 #define VIDEO_OPTIONS_ITEMS 7
 int		video_cursor_table[] = {48, 56, 64, 72, 80, 96, 104};
 int		video_options_cursor = 0;
@@ -1579,13 +1554,20 @@ void VID_Menu_RebuildBppList (void)
 		}
 	}
 
+	//if there are no valid fullscreen bpps for this width/height, just pick one
+	if (vid_menu_numbpps == 0)
+	{
+		Cvar_SetValue ("vid_bpp",(float)modelist[0].bpp);
+		return;
+	}
+
 	//if vid_bpp is not in the new list, change vid_bpp
 	for (i=0;i<vid_menu_numbpps;i++)
 		if (vid_menu_bpps[i] == (int)(vid_bpp.value))
 			break;
 
 	if (i==vid_menu_numbpps)
-		Cvar_Set ("vid_bpp",va("%i",vid_menu_bpps[0]));
+		Cvar_SetValue ("vid_bpp",(float)vid_menu_bpps[0]);
 }
 
 /*
@@ -1620,8 +1602,8 @@ void VID_Menu_ChooseNextMode (int dir)
 			i = vid_menu_nummodes-1;
 	}
 
-	Cvar_Set ("vid_width",va("%i",vid_menu_modes[i].width));
-	Cvar_Set ("vid_height",va("%i",vid_menu_modes[i].height));
+	Cvar_SetValue ("vid_width",(float)vid_menu_modes[i].width);
+	Cvar_SetValue ("vid_height",(float)vid_menu_modes[i].height);
 	VID_Menu_RebuildBppList ();
 }
 
@@ -1655,7 +1637,7 @@ void VID_Menu_ChooseNextBpp (int dir)
 			i = vid_menu_numbpps-1;
 	}
 
-	Cvar_Set ("vid_bpp",va("%i",vid_menu_bpps[i]));
+	Cvar_SetValue ("vid_bpp",(float)vid_menu_bpps[i]);
 }
 
 /*
@@ -1688,7 +1670,7 @@ void VID_Menu_ChooseNextRate (int dir)
 			i = vid_menu_numrates-1;
 	}
 
-	Cvar_Set ("vid_refreshrate",va("%i",vid_menu_rates[i]));
+	Cvar_SetValue ("vid_refreshrate",(float)vid_menu_rates[i]);
 }
 
 /*
@@ -1725,13 +1707,13 @@ void VID_MenuKey (int key)
 		switch (video_options_cursor)
 		{
 		case 0:
-			VID_Menu_ChooseNextMode (-1);
+			VID_Menu_ChooseNextMode (1);
 			break;
 		case 1:
-			VID_Menu_ChooseNextBpp (-1);
+			VID_Menu_ChooseNextBpp (1);
 			break;
 		case 2:
-			VID_Menu_ChooseNextRate (-1);
+			VID_Menu_ChooseNextRate (1);
 			break;
 		case 3:
 			Cbuf_AddText ("toggle vid_fullscreen\n");
@@ -1751,13 +1733,13 @@ void VID_MenuKey (int key)
 		switch (video_options_cursor)
 		{
 		case 0:
-			VID_Menu_ChooseNextMode (1);
+			VID_Menu_ChooseNextMode (-1);
 			break;
 		case 1:
-			VID_Menu_ChooseNextBpp (1);
+			VID_Menu_ChooseNextBpp (-1);
 			break;
 		case 2:
-			VID_Menu_ChooseNextRate (1);
+			VID_Menu_ChooseNextRate (-1);
 			break;
 		case 3:
 			Cbuf_AddText ("toggle vid_fullscreen\n");
@@ -1819,7 +1801,7 @@ void VID_MenuDraw (void)
 {
 	int i = 0;
 	qpic_t *p;
-	char *title;
+	const char *title;
 
 	M_DrawTransPic (16, 4, Draw_CachePic ("gfx/qplaque.lmp"));
 
@@ -1832,39 +1814,39 @@ void VID_MenuDraw (void)
 	M_PrintWhite ((320-8*strlen(title))/2, 32, title);
 
 	// options
-	M_Print (16, video_cursor_table[i], "            Video mode");
-	M_Print (216, video_cursor_table[i], va("%ix%i", (int)vid_width.value, (int)vid_height.value));
+	M_Print (16, video_cursor_table[i], "        Video mode");
+	M_Print (184, video_cursor_table[i], va("%ix%i", (int)vid_width.value, (int)vid_height.value));
 	i++;
 
-	M_Print (16, video_cursor_table[i], "           Color depth");
-	M_Print (216, video_cursor_table[i], va("%i", (int)vid_bpp.value));
+	M_Print (16, video_cursor_table[i], "       Color depth");
+	M_Print (184, video_cursor_table[i], va("%i", (int)vid_bpp.value));
 	i++;
 
-	M_Print (16, video_cursor_table[i], "          Refresh rate");
-//	M_Print (216, video_cursor_table[i], va("%i Hz", (int)vid_refreshrate.value)); refresh rates are disabled for now -- kristian
-	M_Print (216, video_cursor_table[i], "N/A");
+	M_Print (16, video_cursor_table[i], "      Refresh rate");
+//	M_Print (184, video_cursor_table[i], va("%i Hz", (int)vid_refreshrate.value)); refresh rates are disabled for now -- kristian
+	M_Print (184, video_cursor_table[i], "N/A");
 	i++;
 
-	M_Print (16, video_cursor_table[i], "            Fullscreen");
-	M_DrawCheckbox (216, video_cursor_table[i], (int)vid_fullscreen.value);
+	M_Print (16, video_cursor_table[i], "        Fullscreen");
+	M_DrawCheckbox (184, video_cursor_table[i], (int)vid_fullscreen.value);
 	i++;
 
 	// added vsync to the video menu -- kristian
-	M_Print (16, video_cursor_table[i], "         Vertical Sync");
+	M_Print (16, video_cursor_table[i], "     Vertical Sync");
 	if (gl_swap_control)
-		M_DrawCheckbox (216, video_cursor_table[i], (int)vid_vsync.value);
+		M_DrawCheckbox (184, video_cursor_table[i], (int)vid_vsync.value);
 	else
-		M_Print (216, video_cursor_table[i], "N/A");
+		M_Print (184, video_cursor_table[i], "N/A");
 
 	i++;
 
-	M_Print (16, video_cursor_table[i], "          Test changes");
+	M_Print (16, video_cursor_table[i], "      Test changes");
 	i++;
 
-	M_Print (16, video_cursor_table[i], "         Apply changes");
+	M_Print (16, video_cursor_table[i], "     Apply changes");
 
 	// cursor
-	M_DrawCharacter (200, video_cursor_table[video_options_cursor], 12+((int)(realtime*4)&1));
+	M_DrawCharacter (168, video_cursor_table[video_options_cursor], 12+((int)(realtime*4)&1));
 
 	// notes          "345678901234567890123456789012345678"
 //	M_Print (16, 172, "Windowed modes always use the desk- ");
