@@ -22,6 +22,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 // host.c -- coordinates spawning and killing of local servers
 
 #include "quakedef.h"
+#include <setjmp.h>
 
 /*
 
@@ -80,6 +81,9 @@ cvar_t	temp1 = {"temp1","0"};
 
 cvar_t devstats = {"devstats","0"}; //johnfitz -- track developer statistics that vary every frame
 
+devstats_t dev_stats, dev_peakstats;
+overflowtimes_t dev_overflows; //this stores the last time overflow messages were displayed, not the last time overflows occured
+
 /*
 ================
 Max_Edicts_f -- johnfitz
@@ -105,13 +109,13 @@ void Max_Edicts_f (void)
 Host_EndGame
 ================
 */
-void Host_EndGame (char *message, ...)
+void Host_EndGame (const char *message, ...)
 {
 	va_list		argptr;
 	char		string[1024];
 
 	va_start (argptr,message);
-	vsprintf (string,message,argptr);
+	q_vsnprintf (string, sizeof(string), message, argptr);
 	va_end (argptr);
 	Con_DPrintf ("Host_EndGame: %s\n",string);
 
@@ -136,7 +140,7 @@ Host_Error
 This shuts down both the client and server
 ================
 */
-void Host_Error (char *error, ...)
+void Host_Error (const char *error, ...)
 {
 	va_list		argptr;
 	char		string[1024];
@@ -149,7 +153,7 @@ void Host_Error (char *error, ...)
 	SCR_EndLoadingPlaque ();		// reenable screen updates
 
 	va_start (argptr,error);
-	vsprintf (string,error,argptr);
+	q_vsnprintf (string, sizeof(string), error, argptr);
 	va_end (argptr);
 	Con_Printf ("Host_Error: %s\n",string);
 
@@ -211,7 +215,7 @@ void	Host_FindMaxClients (void)
 	svs.maxclientslimit = svs.maxclients;
 	if (svs.maxclientslimit < 4)
 		svs.maxclientslimit = 4;
-	svs.clients = Hunk_AllocName (svs.maxclientslimit*sizeof(client_t), "clients");
+	svs.clients = (struct client_s *) Hunk_AllocName (svs.maxclientslimit*sizeof(client_t), "clients");
 
 	if (svs.maxclients > 1)
 		Cvar_SetValue ("deathmatch", 1.0);
@@ -321,13 +325,13 @@ Sends text across to be displayed
 FIXME: make this just a stuffed echo?
 =================
 */
-void SV_ClientPrintf (char *fmt, ...)
+void SV_ClientPrintf (const char *fmt, ...)
 {
 	va_list		argptr;
 	char		string[1024];
 
 	va_start (argptr,fmt);
-	vsprintf (string, fmt,argptr);
+	q_vsnprintf (string, sizeof(string), fmt,argptr);
 	va_end (argptr);
 
 	MSG_WriteByte (&host_client->message, svc_print);
@@ -341,14 +345,14 @@ SV_BroadcastPrintf
 Sends text to all active clients
 =================
 */
-void SV_BroadcastPrintf (char *fmt, ...)
+void SV_BroadcastPrintf (const char *fmt, ...)
 {
 	va_list		argptr;
 	char		string[1024];
 	int			i;
 
 	va_start (argptr,fmt);
-	vsprintf (string, fmt,argptr);
+	q_vsnprintf (string, sizeof(string), fmt, argptr);
 	va_end (argptr);
 
 	for (i=0 ; i<svs.maxclients ; i++)
@@ -366,13 +370,13 @@ Host_ClientCommands
 Send text over to the client to be executed
 =================
 */
-void Host_ClientCommands (char *fmt, ...)
+void Host_ClientCommands (const char *fmt, ...)
 {
 	va_list		argptr;
 	char		string[1024];
 
 	va_start (argptr,fmt);
-	vsprintf (string, fmt,argptr);
+	q_vsnprintf (string, sizeof(string), fmt, argptr);
 	va_end (argptr);
 
 	MSG_WriteByte (&host_client->message, svc_stufftext);
@@ -497,7 +501,7 @@ void Host_ShutdownServer(qboolean crash)
 	buf.maxsize = 4;
 	buf.cursize = 0;
 	MSG_WriteByte(&buf, svc_disconnect);
-	count = NET_SendToAll(&buf, 5);
+	count = NET_SendToAll(&buf, 5.0);
 	if (count)
 		Con_Printf("Host_ShutdownServer: NET_SendToAll failed for %u clients\n", count);
 
@@ -810,21 +814,20 @@ void Host_Init (quakeparms_t *parms)
 	Memory_Init (parms->membase, parms->memsize);
 	Cbuf_Init ();
 	Cmd_Init ();
+	LOG_Init (parms);
 	Cvar_Init (); //johnfitz
-	V_Init ();
-	Chase_Init ();
 	COM_Init (parms->basedir);
 	Host_InitLocal ();
 	W_LoadWadFile (); //johnfitz -- filename is now hard-coded for honesty
-	Key_Init ();
-	Con_Init ();
-	M_Init ();
+	if (cls.state != ca_dedicated)
+	{
+		Key_Init ();
+		Con_Init ();
+	}
 	PR_Init ();
 	Mod_Init ();
 	NET_Init ();
 	SV_Init ();
-	ExtraMaps_Init (); //johnfitz
-	Modlist_Init (); //johnfitz
 
 	Con_Printf ("Exe: "__TIME__" "__DATE__"\n");
 	Con_Printf ("%4.1f megabyte heap\n",parms->memsize/ (1024*1024.0));
@@ -835,6 +838,11 @@ void Host_Init (quakeparms_t *parms)
 		if (!host_colormap)
 			Sys_Error ("Couldn't load gfx/colormap.lmp");
 
+		V_Init ();
+		Chase_Init ();
+		M_Init ();
+		ExtraMaps_Init (); //johnfitz
+		Modlist_Init (); //johnfitz
 		VID_Init ();
 		IN_Init (); // moved here, SDL inits the input system with the video system -- kristian
 		TexMgr_Init (); //johnfitz
@@ -845,13 +853,21 @@ void Host_Init (quakeparms_t *parms)
 		CDAudio_Init ();
 		Sbar_Init ();
 		CL_Init ();
+
+		Cbuf_InsertText ("exec quake.rc\n");
+	//	Cbuf_InsertText ("exec fitzquake.rc\n"); //johnfitz (inserted second so it'll be executed first)
+
+	// johnfitz -- in case the vid mode was locked during vid_init, we can unlock it now.
+		// note: two leading newlines because the command buffer swallows one of them.
+		Cbuf_AddText ("\n\nvid_unlock\n");
+	}
+	else if (COM_CheckParm ("-cd"))
+	{
+		// Allow "qs -dedicated -cd" to work as a standalone cd player ;>
+		// Useful, because of ubiquitous nature of SDL
+		S_Init ();
+		CDAudio_Init ();
 	}
-
-	Cbuf_InsertText ("exec quake.rc\n");
-//	Cbuf_InsertText ("exec fitzquake.rc\n"); //johnfitz (inserted second so it'll be executed first)
-
-	Cbuf_AddText ("\n\nvid_unlock\n"); //johnfitz -- in case the vid mode was locked during vid_init, we can unlock it now.
-	//note: added two newlines to the front becuase the command buffer swallows one of them.
 
 	Hunk_AllocName (0, "-HOST_HUNKLEVEL-");
 	host_hunklevel = Hunk_LowMark ();
@@ -859,6 +875,15 @@ void Host_Init (quakeparms_t *parms)
 	host_initialized = true;
 
 	Con_Printf ("\n========= Quake Initialized =========\n\n"); //johnfitz - was Sys_Printf
+
+	if (cls.state == ca_dedicated)
+	{
+		Cbuf_AddText ("exec autoexec.cfg\n");
+		Cbuf_AddText ("stuffcmds");
+		Cbuf_Execute ();
+		if (!sv.active)
+			Cbuf_AddText ("map start\n");
+	}
 }
 
 
@@ -886,14 +911,23 @@ void Host_Shutdown(void)
 
 	Host_WriteConfiguration ();
 
-	CDAudio_Shutdown ();
 	NET_Shutdown ();
-	S_Shutdown();
 
 	if (cls.state != ca_dedicated)
 	{
-        IN_Shutdown (); // input is only initialized in Host_Init if we're not dedicated -- kristian
+		if (con_initialized)
+			History_Shutdown ();
+		CDAudio_Shutdown ();
+		S_Shutdown ();
+		IN_Shutdown (); // input is only initialized in Host_Init if we're not dedicated -- kristian
 		VID_Shutdown();
 	}
+	else if (COM_CheckParm ("-cd"))
+	{
+		CDAudio_Shutdown ();
+		S_Shutdown ();
+	}
+
+	LOG_Close ();
 }
 
