@@ -20,8 +20,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
+#include "q_stdinc.h"
+#include "arch_def.h"
+#include "net_sys.h"	/* for net_defs.h */
 #include <dirent.h>
 #include "quakedef.h"
+#include "net_defs.h"	/* for struct qsocket_s details */
 
 extern cvar_t	pausable;
 
@@ -37,8 +41,6 @@ Host_Quit_f
 ==================
 */
 
-extern void M_Menu_Quit_f (void);
-
 void Host_Quit_f (void)
 {
 	if (key_dest != key_console && cls.state != ca_dedicated)
@@ -59,28 +61,28 @@ void Host_Quit_f (void)
 // Declarations shared with common.c:
 typedef struct
 {
-	char    name[MAX_QPATH];
-	int             filepos, filelen;
+	char	name[MAX_QPATH];
+	int	filepos, filelen;
 } packfile_t;
 
 typedef struct pack_s
 {
-	char    filename[MAX_OSPATH];
-	int             handle;
-	int             numfiles;
-	packfile_t      *files;
+	char	filename[MAX_OSPATH];
+	int		handle;
+	int		numfiles;
+	packfile_t	*files;
 } pack_t;
 
 typedef struct searchpath_s
 {
-	char    filename[MAX_OSPATH];
-	pack_t  *pack;          // only one of filename / pack will be used
+	char	filename[MAX_OSPATH];
+	pack_t	*pack;		// only one of filename / pack will be used
 	struct searchpath_s *next;
 } searchpath_t;
 
 extern qboolean com_modified;
 extern searchpath_t *com_searchpaths;
-pack_t *COM_LoadPackFile (char *packfile);
+pack_t *COM_LoadPackFile (const char *packfile);
 
 // Kill all the search packs until the game path is found. Kill it, then return
 // the next path to it.
@@ -170,7 +172,7 @@ void Host_Game_f (void)
 
 		if (Q_strcasecmp(Cmd_Argv(1), GAMENAME)) //game is not id1
 		{
-			search = Z_Malloc(sizeof(searchpath_t));
+			search = (searchpath_t *) Z_Malloc(sizeof(searchpath_t));
 			strcpy (search->filename, pakfile);
 			search->next = com_searchpaths;
 			com_searchpaths = search;
@@ -181,8 +183,8 @@ void Host_Game_f (void)
 				sprintf (pakfile, "%s/pak%i.pak", com_gamedir, i);
 				pak = COM_LoadPackFile (pakfile);
 				if (!pak)
-				break;
-				search = Z_Malloc(sizeof(searchpath_t));
+					break;
+				search = (searchpath_t *) Z_Malloc(sizeof(searchpath_t));
 				search->pack = pak;
 				search->next = com_searchpaths;
 				com_searchpaths = search;
@@ -212,59 +214,63 @@ void Host_Game_f (void)
 
 typedef struct extralevel_s
 {
-	char				name[32];
+	char			name[32];
 	struct extralevel_s	*next;
 } extralevel_t;
 
 extralevel_t	*extralevels;
 
-void ExtraMaps_Add (char *name)
+void ExtraMaps_Add (const char *name)
 {
 	extralevel_t	*level,*cursor,*prev;
 
-	//ingore duplicate
+	// ignore duplicate
 	for (level = extralevels; level; level = level->next)
+	{
 		if (!Q_strcmp (name, level->name))
 			return;
+	}
 
-	level = Z_Malloc(sizeof(extralevel_t));
+	level = (extralevel_t *) Z_Malloc(sizeof(extralevel_t));
 	strcpy (level->name, name);
 
-	//insert each entry in alphabetical order
-    if (extralevels == NULL || Q_strcasecmp(level->name, extralevels->name) > 0) //insert at front
+	// insert each entry in alphabetical order
+	if (extralevels == NULL ||
+	    Q_strcasecmp(level->name, extralevels->name) < 0) //insert at front
 	{
-        level->next = extralevels;
-        extralevels = level;
-    }
-    else //insert later
+		level->next = extralevels;
+		extralevels = level;
+	}
+	else //insert later
 	{
-        prev = extralevels;
-        cursor = extralevels->next;
-        while (cursor && (Q_strcasecmp(level->name, cursor->name) < 0))
+		prev = extralevels;
+		cursor = extralevels->next;
+		while (cursor && (Q_strcasecmp(level->name, cursor->name) > 0))
 		{
-            prev = cursor;
-            cursor = cursor->next;
-        }
-        level->next = prev->next;
-        prev->next = level;
-    }
+			prev = cursor;
+			cursor = cursor->next;
+		}
+		level->next = prev->next;
+		prev->next = level;
+	}
 }
 
 void ExtraMaps_Init (void)
 {
 	DIR		*dir_p;
 	struct dirent	*dir_t;
-	char			filestring[MAX_OSPATH];
-	char			mapname[32];
-	char			ignorepakdir[32];
+	char		filestring[MAX_OSPATH];
+	char		mapname[32];
+	char		ignorepakdir[32];
 	searchpath_t	*search;
 	pack_t		*pak;
 	int		i;
 
-	//we don't want to list the maps in id1 pakfiles, becuase these are not "add-on" levels
+	// we don't want to list the maps in id1 pakfiles,
+	// because these are not "add-on" levels
 	sprintf (ignorepakdir, "/%s/", GAMENAME);
 
-	for (search = com_searchpaths ; search ; search = search->next)
+	for (search = com_searchpaths; search; search = search->next)
 	{
 		if (*search->filename) //directory
 		{
@@ -283,14 +289,14 @@ void ExtraMaps_Init (void)
 		}
 		else //pakfile
 		{
-			if (!strstr(search->pack->filename, ignorepakdir)) //don't list standard id maps
-			{
-				for (i=0, pak=search->pack; i<pak->numfiles ; i++)
+			if (!strstr(search->pack->filename, ignorepakdir))
+			{ //don't list standard id maps
+				for (i = 0, pak = search->pack; i < pak->numfiles; i++)
 				{
 					if (strstr(pak->files[i].name, ".bsp"))
 					{
-						if (pak->files[i].filelen > 32*1024) // don't list files under 32k (ammo boxes etc)
-						{
+						if (pak->files[i].filelen > 32*1024)
+						{ // don't list files under 32k (ammo boxes etc)
 							COM_StripExtension(pak->files[i].name + 5, mapname);
 							ExtraMaps_Add (mapname);
 						}
@@ -329,7 +335,7 @@ void Host_Maps_f (void)
 	int i;
 	extralevel_t	*level;
 
-	for (level=extralevels, i=0; level; level=level->next, i++)
+	for (level = extralevels, i = 0; level; level = level->next, i++)
 		Con_SafePrintf ("   %s\n", level->name);
 
 	if (i)
@@ -344,42 +350,45 @@ void Host_Maps_f (void)
 
 typedef struct mod_s
 {
-	char			name[MAX_OSPATH];
+	char	name[MAX_OSPATH];
 	struct mod_s	*next;
 } mod_t;
 
 mod_t	*modlist;
 
-void Modlist_Add (char *name)
+void Modlist_Add (const char *name)
 {
 	mod_t	*mod,*cursor,*prev;
 
 	//ingore duplicate
 	for (mod = modlist; mod; mod = mod->next)
+	{
 		if (!Q_strcmp (name, mod->name))
 			return;
+	}
 
-	mod = Z_Malloc(sizeof(mod_t));
+	mod = (mod_t *) Z_Malloc(sizeof(mod_t));
 	strcpy (mod->name, name);
 
 	//insert each entry in alphabetical order
-    if (modlist == NULL || Q_strcasecmp(mod->name, modlist->name) > 0) //insert at front
+	if (modlist == NULL ||
+	    Q_strcasecmp(mod->name, modlist->name) < 0) //insert at front
 	{
-        mod->next = modlist;
-        modlist = mod;
-    }
-    else //insert later
+		mod->next = modlist;
+		modlist = mod;
+	}
+	else //insert later
 	{
-        prev = modlist;
-        cursor = modlist->next;
-        while (cursor && (Q_strcasecmp(mod->name, cursor->name) < 0))
+		prev = modlist;
+		cursor = modlist->next;
+		while (cursor && (Q_strcasecmp(mod->name, cursor->name) > 0))
 		{
-            prev = cursor;
-            cursor = cursor->next;
-        }
-        mod->next = prev->next;
-        prev->next = mod;
-    }
+			prev = cursor;
+			cursor = cursor->next;
+		}
+		mod->next = prev->next;
+		prev->next = mod;
+	}
 }
 
 void Modlist_Init (void)
@@ -387,7 +396,7 @@ void Modlist_Init (void)
 	DIR		*dir_p, *mod_dir_p;
 	struct dirent	*dir_t, *mod_dir_t;
 	qboolean	progs_found, pak_found;
-	char			dir_string[MAX_OSPATH], mod_dir_string[MAX_OSPATH];
+	char		dir_string[MAX_OSPATH], mod_dir_string[MAX_OSPATH];
 	int		i;
 
 	i = COM_CheckParm ("-basedir");
@@ -415,7 +424,7 @@ void Modlist_Init (void)
 		{
 			if ((strcmp(mod_dir_t->d_name, ".") == 0) || (strcmp(mod_dir_t->d_name, "..") == 0))
 				continue;
-			if (Q_strcasecmp(mod_dir_t->d_name, "progs.dat") != -1)
+			if (Q_strcasecmp(mod_dir_t->d_name, "progs.dat") == 0)
 				progs_found = true;
 			if (strstr(mod_dir_t->d_name, ".pak") || strstr(mod_dir_t->d_name, ".PAK"))
 				pak_found = true;
@@ -461,19 +470,15 @@ Host_Mapname_f -- johnfitz
 */
 void Host_Mapname_f (void)
 {
-	char name[MAX_QPATH];
-
 	if (sv.active)
 	{
-		COM_StripExtension (sv.worldmodel->name + 5, name);
-		Con_Printf ("\"mapname\" is \"%s\"\n", name);
+		Con_Printf ("\"mapname\" is \"%s\"\n", sv.name);
 		return;
 	}
 
 	if (cls.state == ca_connected)
 	{
-		COM_StripExtension (cl.worldmodel->name + 5, name);
-		Con_Printf ("\"mapname\" is \"%s\"\n", name);
+		Con_Printf ("\"mapname\" is \"%s\"\n", cl.mapname);
 		return;
 	}
 
@@ -492,7 +497,7 @@ void Host_Status_f (void)
 	int			minutes;
 	int			hours = 0;
 	int			j;
-	void		(*print) (char *fmt, ...);
+	void		(*print_fn) (const char *fmt, ...) __fp_attribute__((__format__(__printf__,1,2)));
 
 	if (cmd_source == src_command)
 	{
@@ -501,20 +506,20 @@ void Host_Status_f (void)
 			Cmd_ForwardToServer ();
 			return;
 		}
-		print = Con_Printf;
+		print_fn = Con_Printf;
 	}
 	else
-		print = SV_ClientPrintf;
+		print_fn = SV_ClientPrintf;
 
-	print ("host:    %s\n", Cvar_VariableString ("hostname"));
-	print ("version: %4.2f\n", VERSION);
+	print_fn ("host:    %s\n", Cvar_VariableString ("hostname"));
+	print_fn ("version: %4.2f\n", VERSION);
 	if (tcpipAvailable)
-		print ("tcp/ip:  %s\n", my_tcpip_address);
+		print_fn ("tcp/ip:  %s\n", my_tcpip_address);
 	if (ipxAvailable)
-		print ("ipx:     %s\n", my_ipx_address);
-	print ("map:     %s\n", sv.name);
-	print ("players: %i active (%i max)\n\n", net_activeconnections, svs.maxclients);
-	for (j=0, client = svs.clients ; j<svs.maxclients ; j++, client++)
+		print_fn ("ipx:     %s\n", my_ipx_address);
+	print_fn ("map:     %s\n", sv.name);
+	print_fn ("players: %i active (%i max)\n\n", net_activeconnections, svs.maxclients);
+	for (j = 0, client = svs.clients; j < svs.maxclients; j++, client++)
 	{
 		if (!client->active)
 			continue;
@@ -529,8 +534,8 @@ void Host_Status_f (void)
 		}
 		else
 			hours = 0;
-		print ("#%-2u %-16.16s  %3i  %2i:%02i:%02i\n", j+1, client->name, (int)client->edict->v.frags, hours, minutes, seconds);
-		print ("   %s\n", client->netconnection->address);
+		print_fn ("#%-2u %-16.16s  %3i  %2i:%02i:%02i\n", j+1, client->name, (int)client->edict->v.frags, hours, minutes, seconds);
+		print_fn ("   %s\n", client->netconnection->address);
 	}
 }
 
@@ -744,7 +749,7 @@ Host_Ping_f
 void Host_Ping_f (void)
 {
 	int		i, j;
-	float	total;
+	float		total;
 	client_t	*client;
 
 	if (cmd_source == src_command)
@@ -754,12 +759,12 @@ void Host_Ping_f (void)
 	}
 
 	SV_ClientPrintf ("Client ping times:\n");
-	for (i=0, client = svs.clients ; i<svs.maxclients ; i++, client++)
+	for (i = 0, client = svs.clients; i < svs.maxclients; i++, client++)
 	{
 		if (!client->active)
 			continue;
 		total = 0;
-		for (j=0 ; j<NUM_PING_TIMES ; j++)
+		for (j = 0; j < NUM_PING_TIMES; j++)
 			total+=client->ping_times[j];
 		total /= NUM_PING_TIMES;
 		SV_ClientPrintf ("%4i %s\n", (int)(total*1000), client->name);
@@ -787,7 +792,7 @@ command from the console.  Active clients are kicked off.
 void Host_Map_f (void)
 {
 	int		i;
-	char	name[MAX_QPATH];
+	char	name[MAX_QPATH], *p;
 
 	if (Cmd_Argc() < 2)	//no map name given
 	{
@@ -817,20 +822,17 @@ void Host_Map_f (void)
 	CL_Disconnect ();
 	Host_ShutdownServer(false);
 
-	IN_Activate();
+	if (cls.state != ca_dedicated)
+		IN_Activate();
 	key_dest = key_game;			// remove console or menu
 	SCR_BeginLoadingPlaque ();
 
-	cls.mapstring[0] = 0;
-	for (i=0 ; i<Cmd_Argc() ; i++)
-	{
-		strcat (cls.mapstring, Cmd_Argv(i));
-		strcat (cls.mapstring, " ");
-	}
-	strcat (cls.mapstring, "\n");
-
 	svs.serverflags = 0;			// haven't completed an episode yet
 	strcpy (name, Cmd_Argv(1));
+	// remove (any) trailing ".bsp" from mapname S.A.
+	p = strstr(name, ".bsp");
+	if (p && p[4] == '\0')
+		*p = '\0';
 	SV_SpawnServer (name);
 	if (!sv.active)
 		return;
@@ -839,7 +841,7 @@ void Host_Map_f (void)
 	{
 		strcpy (cls.spawnparms, "");
 
-		for (i=2 ; i<Cmd_Argc() ; i++)
+		for (i = 2; i < Cmd_Argc(); i++)
 		{
 			strcat (cls.spawnparms, Cmd_Argv(i));
 			strcat (cls.spawnparms, " ");
@@ -878,7 +880,8 @@ void Host_Changelevel_f (void)
 		Host_Error ("cannot find map %s", level);
 	//johnfitz
 
-	IN_Activate();	// -- S.A.
+	if (cls.state != ca_dedicated)
+		IN_Activate();	// -- S.A.
 	key_dest = key_game;	// remove console or menu
 	SV_SaveSpawnparms ();
 	strcpy (level, Cmd_Argv(1));
@@ -965,15 +968,17 @@ void Host_SavegameComment (char *text)
 	int		i;
 	char	kills[20];
 
-	for (i=0 ; i<SAVEGAME_COMMENT_LENGTH ; i++)
+	for (i = 0; i < SAVEGAME_COMMENT_LENGTH; i++)
 		text[i] = ' ';
 	memcpy (text, cl.levelname, min(strlen(cl.levelname),22)); //johnfitz -- only copy 22 chars.
 	sprintf (kills,"kills:%3i/%3i", cl.stats[STAT_MONSTERS], cl.stats[STAT_TOTALMONSTERS]);
 	memcpy (text+22, kills, strlen(kills));
 // convert space to _ to make stdio happy
-	for (i=0 ; i<SAVEGAME_COMMENT_LENGTH ; i++)
+	for (i = 0; i < SAVEGAME_COMMENT_LENGTH; i++)
+	{
 		if (text[i] == ' ')
 			text[i] = '_';
+	}
 	text[SAVEGAME_COMMENT_LENGTH] = '\0';
 }
 
@@ -987,7 +992,7 @@ void Host_Savegame_f (void)
 {
 	char	name[256];
 	FILE	*f;
-	int		i;
+	int	i;
 	char	comment[SAVEGAME_COMMENT_LENGTH+1];
 
 	if (cmd_source != src_command)
@@ -1046,7 +1051,7 @@ void Host_Savegame_f (void)
 	fprintf (f, "%i\n", SAVEGAME_VERSION);
 	Host_SavegameComment (comment);
 	fprintf (f, "%s\n", comment);
-	for (i=0 ; i<NUM_SPAWN_PARMS ; i++)
+	for (i = 0; i < NUM_SPAWN_PARMS; i++)
 		fprintf (f, "%f\n", svs.clients->spawn_parms[i]);
 	fprintf (f, "%d\n", current_skill);
 	fprintf (f, "%s\n", sv.name);
@@ -1054,7 +1059,7 @@ void Host_Savegame_f (void)
 
 // write the light styles
 
-	for (i=0 ; i<MAX_LIGHTSTYLES ; i++)
+	for (i = 0; i < MAX_LIGHTSTYLES; i++)
 	{
 		if (sv.lightstyles[i])
 			fprintf (f, "%s\n", sv.lightstyles[i]);
@@ -1064,7 +1069,7 @@ void Host_Savegame_f (void)
 
 
 	ED_WriteGlobals (f);
-	for (i=0 ; i<sv.num_edicts ; i++)
+	for (i = 0; i < sv.num_edicts; i++)
 	{
 		ED_Write (f, EDICT_NUM(i));
 		fflush (f);
@@ -1085,12 +1090,13 @@ void Host_Loadgame_f (void)
 	FILE	*f;
 	char	mapname[MAX_QPATH];
 	float	time, tfloat;
-	char	str[32768], *start;
-	int		i, r;
+	char	str[32768];
+	const char  *start;
+	int	i, r;
 	edict_t	*ent;
-	int		entnum;
-	int		version;
-	float			spawn_parms[NUM_SPAWN_PARMS];
+	int	entnum;
+	int	version;
+	float	spawn_parms[NUM_SPAWN_PARMS];
 
 	if (cmd_source != src_command)
 		return;
@@ -1126,7 +1132,7 @@ void Host_Loadgame_f (void)
 		return;
 	}
 	fscanf (f, "%s\n", str);
-	for (i=0 ; i<NUM_SPAWN_PARMS ; i++)
+	for (i = 0; i < NUM_SPAWN_PARMS; i++)
 		fscanf (f, "%f\n", &spawn_parms[i]);
 // this silliness is so we can load 1.06 save files, which have float skill values
 	fscanf (f, "%f\n", &tfloat);
@@ -1150,18 +1156,17 @@ void Host_Loadgame_f (void)
 
 // load the light styles
 
-	for (i=0 ; i<MAX_LIGHTSTYLES ; i++)
+	for (i = 0; i < MAX_LIGHTSTYLES; i++)
 	{
 		fscanf (f, "%s\n", str);
-		sv.lightstyles[i] = Hunk_Alloc (strlen(str)+1);
-		strcpy (sv.lightstyles[i], str);
+		sv.lightstyles[i] = (const char *)Hunk_Strdup (str, "lightstyles");
 	}
 
 // load the edicts out of the savegame file
 	entnum = -1;		// -1 is the globals
 	while (!feof(f))
 	{
-		for (i=0 ; i<sizeof(str)-1 ; i++)
+		for (i = 0; i < sizeof(str) - 1; i++)
 		{
 			r = fgetc (f);
 			if (r == EOF || !r)
@@ -1173,7 +1178,7 @@ void Host_Loadgame_f (void)
 				break;
 			}
 		}
-		if (i == sizeof(str)-1)
+		if (i == sizeof(str) - 1)
 			Sys_Error ("Loadgame buffer overflow");
 		str[i] = 0;
 		start = str;
@@ -1208,7 +1213,7 @@ void Host_Loadgame_f (void)
 
 	fclose (f);
 
-	for (i=0 ; i<NUM_SPAWN_PARMS ; i++)
+	for (i = 0; i < NUM_SPAWN_PARMS; i++)
 		svs.clients->spawn_parms[i] = spawn_parms[i];
 
 	if (cls.state != ca_dedicated)
@@ -1227,7 +1232,7 @@ Host_Name_f
 */
 void Host_Name_f (void)
 {
-	char	*newName;
+	char	newName[32];
 
 	if (Cmd_Argc () == 1)
 	{
@@ -1235,10 +1240,10 @@ void Host_Name_f (void)
 		return;
 	}
 	if (Cmd_Argc () == 2)
-		newName = Cmd_Argv(1);
+		Q_strncpy(newName, Cmd_Argv(1), sizeof(newName)-1);
 	else
-		newName = Cmd_Args();
-	newName[15] = 0;
+		Q_strncpy(newName, Cmd_Args(), sizeof(newName)-1);
+	newName[15] = 0;	// client_t structure actually says name[32].
 
 	if (cmd_source == src_command)
 	{
@@ -1251,8 +1256,10 @@ void Host_Name_f (void)
 	}
 
 	if (host_client->name[0] && strcmp(host_client->name, "unconnected") )
+	{
 		if (Q_strcmp(host_client->name, newName) != 0)
 			Con_Printf ("%s renamed to %s\n", host_client->name, newName);
+	}
 	Q_strcpy (host_client->name, newName);
 	host_client->edict->v.netname = PR_SetEngineString(host_client->name);
 
@@ -1267,18 +1274,18 @@ void Host_Name_f (void)
 void Host_Version_f (void)
 {
 	Con_Printf ("Quake Version %1.2f\n", VERSION); //johnfitz
-	Con_Printf ("FitzQuake (SDL port) Version %1.2f\n", FITZQUAKE_VERSION); //johnfitz
+	Con_Printf ("QuakeSpasm Version %1.2f.%d\n", FITZQUAKE_VERSION, QUAKESPASM_VER_PATCH); //johnfitz
 	Con_Printf ("Exe: "__TIME__" "__DATE__"\n");
 }
 
 void Host_Say(qboolean teamonly)
 {
-	client_t *client;
-	client_t *save;
 	int		j;
-	char	*p;
-    // removed unsigned keyword -- kristian
-	char	text[64];
+	client_t	*client;
+	client_t	*save;
+	const char	*p;
+	char		text[MAXCMDLINE], *p2;
+	qboolean	quoted;
 	qboolean	fromServer = false;
 
 	if (cmd_source == src_command)
@@ -1302,24 +1309,39 @@ void Host_Say(qboolean teamonly)
 
 	p = Cmd_Args();
 // remove quotes if present
-	if (*p == '"')
+	quoted = false;
+	if (*p == '\"')
 	{
 		p++;
-		p[Q_strlen(p)-1] = 0;
+		quoted = true;
 	}
-
 // turn on color set 1
 	if (!fromServer)
-		sprintf (text, "%c%s: ", 1, save->name);
+		q_snprintf (text, sizeof(text), "\001%s: %s", save->name, p);
 	else
-		sprintf (text, "%c<%s> ", 1, hostname.string);
+		q_snprintf (text, sizeof(text), "\001<%s> %s", hostname.string, p);
 
-	j = sizeof(text) - 2 - Q_strlen(text);  // -2 for /n and null terminator
-	if (Q_strlen(p) > j)
-		p[j] = 0;
-
-	strcat (text, p);
-	strcat (text, "\n");
+// check length & truncate if necessary
+	j = strlen(text);
+	if (j >= sizeof(text) - 1)
+	{
+		text[sizeof(text) - 2] = '\n';
+		text[sizeof(text) - 1] = '\0';
+	}
+	else
+	{
+		p2 = text + j;
+		while ((const char *)p2 > (const char *)text &&
+			(p2[-1] == '\r' || p2[-1] == '\n' || (p2[-1] == '\"' && quoted)) )
+		{
+			if (p2[-1] == '\"' && quoted)
+				quoted = false;
+			p2[-1] = '\0';
+			p2--;
+		}
+		p2[0] = '\n';
+		p2[1] = '\0';
+	}
 
 	for (j = 0, client = svs.clients; j < svs.maxclients; j++, client++)
 	{
@@ -1332,7 +1354,8 @@ void Host_Say(qboolean teamonly)
 	}
 	host_client = save;
 
-	Sys_Printf("%s", &text[1]);
+	if (cls.state == ca_dedicated)
+		Sys_Printf("%s", &text[1]);
 }
 
 
@@ -1350,11 +1373,12 @@ void Host_Say_Team_f(void)
 
 void Host_Tell_f(void)
 {
-	client_t *client;
-	client_t *save;
 	int		j;
-	char	*p;
-	char	text[64];
+	client_t	*client;
+	client_t	*save;
+	const char	*p;
+	char		text[MAXCMDLINE], *p2;
+	qboolean	quoted;
 
 	if (cmd_source == src_command)
 	{
@@ -1365,25 +1389,37 @@ void Host_Tell_f(void)
 	if (Cmd_Argc () < 3)
 		return;
 
-	Q_strcpy(text, host_client->name);
-	Q_strcat(text, ": ");
-
 	p = Cmd_Args();
-
 // remove quotes if present
-	if (*p == '"')
+	quoted = false;
+	if (*p == '\"')
 	{
 		p++;
-		p[Q_strlen(p)-1] = 0;
+		quoted = true;
 	}
+	q_snprintf (text, sizeof(text), "%s: %s", host_client->name, p);
 
 // check length & truncate if necessary
-	j = sizeof(text) - 2 - Q_strlen(text);  // -2 for /n and null terminator
-	if (Q_strlen(p) > j)
-		p[j] = 0;
-
-	strcat (text, p);
-	strcat (text, "\n");
+	j = strlen(text);
+	if (j >= sizeof(text) - 1)
+	{
+		text[sizeof(text) - 2] = '\n';
+		text[sizeof(text) - 1] = '\0';
+	}
+	else
+	{
+		p2 = text + j;
+		while ((const char *)p2 > (const char *)text &&
+			(p2[-1] == '\r' || p2[-1] == '\n' || (p2[-1] == '\"' && quoted)) )
+		{
+			if (p2[-1] == '\"' && quoted)
+				quoted = false;
+			p2[-1] = '\0';
+			p2--;
+		}
+		p2[0] = '\n';
+		p2[1] = '\0';
+	}
 
 	save = host_client;
 	for (j = 0, client = svs.clients; j < svs.maxclients; j++, client++)
@@ -1602,7 +1638,7 @@ void Host_Spawn_f (void)
 	MSG_WriteByte (&host_client->message, svc_time);
 	MSG_WriteFloat (&host_client->message, sv.time);
 
-	for (i=0, client = svs.clients ; i<svs.maxclients ; i++, client++)
+	for (i = 0, client = svs.clients; i < svs.maxclients; i++, client++)
 	{
 		MSG_WriteByte (&host_client->message, svc_updatename);
 		MSG_WriteByte (&host_client->message, i);
@@ -1616,7 +1652,7 @@ void Host_Spawn_f (void)
 	}
 
 // send all current light styles
-	for (i=0 ; i<MAX_LIGHTSTYLES ; i++)
+	for (i = 0; i < MAX_LIGHTSTYLES; i++)
 	{
 		MSG_WriteByte (&host_client->message, svc_lightstyle);
 		MSG_WriteByte (&host_client->message, (char)i);
@@ -1651,7 +1687,7 @@ void Host_Spawn_f (void)
 // with a permanent head tilt
 	ent = EDICT_NUM( 1 + (host_client - svs.clients) );
 	MSG_WriteByte (&host_client->message, svc_setangle);
-	for (i=0 ; i < 2 ; i++)
+	for (i = 0; i < 2; i++)
 		MSG_WriteAngle (&host_client->message, ent->v.angles[i] );
 	MSG_WriteAngle (&host_client->message, 0 );
 
@@ -1690,10 +1726,10 @@ Kicks a user off of the server
 */
 void Host_Kick_f (void)
 {
-	char		*who;
-	char		*message = NULL;
+	const char	*who;
+	const char	*message = NULL;
 	client_t	*save;
-	int			i;
+	int		i;
 	qboolean	byNumber = false;
 
 	if (cmd_source == src_command)
@@ -1782,8 +1818,8 @@ Host_Give_f
 */
 void Host_Give_f (void)
 {
-	char	*t;
-	int		v;
+	const char	*t;
+	int	v;
 	eval_t	*val;
 
 	if (cmd_source == src_command)
@@ -1800,160 +1836,173 @@ void Host_Give_f (void)
 
 	switch (t[0])
 	{
-   case '0':
-   case '1':
-   case '2':
-   case '3':
-   case '4':
-   case '5':
-   case '6':
-   case '7':
-   case '8':
-   case '9':
-      // MED 01/04/97 added hipnotic give stuff
-      if (hipnotic)
-      {
-         if (t[0] == '6')
-         {
-            if (t[1] == 'a')
-               sv_player->v.items = (int)sv_player->v.items | HIT_PROXIMITY_GUN;
-            else
-               sv_player->v.items = (int)sv_player->v.items | IT_GRENADE_LAUNCHER;
-         }
-         else if (t[0] == '9')
-            sv_player->v.items = (int)sv_player->v.items | HIT_LASER_CANNON;
-         else if (t[0] == '0')
-            sv_player->v.items = (int)sv_player->v.items | HIT_MJOLNIR;
-         else if (t[0] >= '2')
-            sv_player->v.items = (int)sv_player->v.items | (IT_SHOTGUN << (t[0] - '2'));
-      }
-      else
-      {
-         if (t[0] >= '2')
-            sv_player->v.items = (int)sv_player->v.items | (IT_SHOTGUN << (t[0] - '2'));
-      }
+	case '0':
+	case '1':
+	case '2':
+	case '3':
+	case '4':
+	case '5':
+	case '6':
+	case '7':
+	case '8':
+	case '9':
+		// MED 01/04/97 added hipnotic give stuff
+		if (hipnotic)
+		{
+		    if (t[0] == '6')
+		    {
+			if (t[1] == 'a')
+			    sv_player->v.items = (int)sv_player->v.items | HIT_PROXIMITY_GUN;
+			else
+			    sv_player->v.items = (int)sv_player->v.items | IT_GRENADE_LAUNCHER;
+		    }
+		    else if (t[0] == '9')
+			sv_player->v.items = (int)sv_player->v.items | HIT_LASER_CANNON;
+		    else if (t[0] == '0')
+			sv_player->v.items = (int)sv_player->v.items | HIT_MJOLNIR;
+		    else if (t[0] >= '2')
+			sv_player->v.items = (int)sv_player->v.items | (IT_SHOTGUN << (t[0] - '2'));
+		}
+		else
+		{
+		    if (t[0] >= '2')
+			sv_player->v.items = (int)sv_player->v.items | (IT_SHOTGUN << (t[0] - '2'));
+		}
 		break;
 
-    case 's':
+	case 's':
 		if (rogue)
 		{
-	        val = GetEdictFieldValue(sv_player, "ammo_shells1");
+		    val = GetEdictFieldValue(sv_player, "ammo_shells1");
 		    if (val)
-			    val->_float = v;
+			val->_float = v;
 		}
+		sv_player->v.ammo_shells = v;
+		break;
 
-        sv_player->v.ammo_shells = v;
-        break;
-    case 'n':
+	case 'n':
 		if (rogue)
 		{
-			val = GetEdictFieldValue(sv_player, "ammo_nails1");
-			if (val)
-			{
-				val->_float = v;
-				if (sv_player->v.weapon <= IT_LIGHTNING)
-					sv_player->v.ammo_nails = v;
-			}
+		    val = GetEdictFieldValue(sv_player, "ammo_nails1");
+		    if (val)
+		    {
+			val->_float = v;
+			if (sv_player->v.weapon <= IT_LIGHTNING)
+			    sv_player->v.ammo_nails = v;
+		    }
 		}
 		else
 		{
-			sv_player->v.ammo_nails = v;
+		    sv_player->v.ammo_nails = v;
 		}
-        break;
-    case 'l':
+		break;
+
+	case 'l':
 		if (rogue)
 		{
-			val = GetEdictFieldValue(sv_player, "ammo_lava_nails");
-			if (val)
-			{
-				val->_float = v;
-				if (sv_player->v.weapon > IT_LIGHTNING)
-					sv_player->v.ammo_nails = v;
-			}
+		    val = GetEdictFieldValue(sv_player, "ammo_lava_nails");
+		    if (val)
+		    {
+			val->_float = v;
+			if (sv_player->v.weapon > IT_LIGHTNING)
+			    sv_player->v.ammo_nails = v;
+		    }
 		}
-        break;
-    case 'r':
+		break;
+
+	case 'r':
 		if (rogue)
 		{
-			val = GetEdictFieldValue(sv_player, "ammo_rockets1");
-			if (val)
-			{
-				val->_float = v;
-				if (sv_player->v.weapon <= IT_LIGHTNING)
-					sv_player->v.ammo_rockets = v;
-			}
+		    val = GetEdictFieldValue(sv_player, "ammo_rockets1");
+		    if (val)
+		    {
+			val->_float = v;
+			if (sv_player->v.weapon <= IT_LIGHTNING)
+			    sv_player->v.ammo_rockets = v;
+		    }
 		}
 		else
 		{
-			sv_player->v.ammo_rockets = v;
+		    sv_player->v.ammo_rockets = v;
 		}
-        break;
-    case 'm':
+		break;
+
+	case 'm':
 		if (rogue)
 		{
-			val = GetEdictFieldValue(sv_player, "ammo_multi_rockets");
-			if (val)
-			{
-				val->_float = v;
-				if (sv_player->v.weapon > IT_LIGHTNING)
-					sv_player->v.ammo_rockets = v;
-			}
+		    val = GetEdictFieldValue(sv_player, "ammo_multi_rockets");
+		    if (val)
+		    {
+			val->_float = v;
+			if (sv_player->v.weapon > IT_LIGHTNING)
+			    sv_player->v.ammo_rockets = v;
+		    }
 		}
-        break;
-    case 'h':
-        sv_player->v.health = v;
-        break;
-    case 'c':
+		break;
+
+	case 'h':
+		sv_player->v.health = v;
+		break;
+
+	case 'c':
 		if (rogue)
 		{
-			val = GetEdictFieldValue(sv_player, "ammo_cells1");
-			if (val)
-			{
-				val->_float = v;
-				if (sv_player->v.weapon <= IT_LIGHTNING)
-					sv_player->v.ammo_cells = v;
-			}
+		    val = GetEdictFieldValue(sv_player, "ammo_cells1");
+		    if (val)
+		    {
+			val->_float = v;
+			if (sv_player->v.weapon <= IT_LIGHTNING)
+			    sv_player->v.ammo_cells = v;
+		    }
 		}
 		else
 		{
-			sv_player->v.ammo_cells = v;
+		    sv_player->v.ammo_cells = v;
 		}
-        break;
-    case 'p':
+		break;
+
+	case 'p':
 		if (rogue)
 		{
-			val = GetEdictFieldValue(sv_player, "ammo_plasma");
-			if (val)
-			{
-				val->_float = v;
-				if (sv_player->v.weapon > IT_LIGHTNING)
-					sv_player->v.ammo_cells = v;
-			}
+		    val = GetEdictFieldValue(sv_player, "ammo_plasma");
+		    if (val)
+		    {
+			val->_float = v;
+			if (sv_player->v.weapon > IT_LIGHTNING)
+			    sv_player->v.ammo_cells = v;
+		    }
 		}
-        break;
+		break;
+
 	//johnfitz -- give armour
-    case 'a':
+	case 'a':
 		if (v > 150)
 		{
-			sv_player->v.armortype = 0.8;
-	        sv_player->v.armorvalue = v;
-			sv_player->v.items = sv_player->v.items - ((int)(sv_player->v.items) & (int)(IT_ARMOR1 | IT_ARMOR2 | IT_ARMOR3)) + IT_ARMOR3;
+		    sv_player->v.armortype = 0.8;
+		    sv_player->v.armorvalue = v;
+		    sv_player->v.items = sv_player->v.items -
+					((int)(sv_player->v.items) & (int)(IT_ARMOR1 | IT_ARMOR2 | IT_ARMOR3)) +
+					IT_ARMOR3;
 		}
 		else if (v > 100)
 		{
-			sv_player->v.armortype = 0.6;
-	        sv_player->v.armorvalue = v;
-			sv_player->v.items = sv_player->v.items - ((int)(sv_player->v.items) & (int)(IT_ARMOR1 | IT_ARMOR2 | IT_ARMOR3)) + IT_ARMOR2;
+		    sv_player->v.armortype = 0.6;
+		    sv_player->v.armorvalue = v;
+		    sv_player->v.items = sv_player->v.items -
+					((int)(sv_player->v.items) & (int)(IT_ARMOR1 | IT_ARMOR2 | IT_ARMOR3)) +
+					IT_ARMOR2;
 		}
 		else if (v >= 0)
 		{
-			sv_player->v.armortype = 0.3;
-	        sv_player->v.armorvalue = v;
-			sv_player->v.items = sv_player->v.items - ((int)(sv_player->v.items) & (int)(IT_ARMOR1 | IT_ARMOR2 | IT_ARMOR3)) + IT_ARMOR1;
+		    sv_player->v.armortype = 0.3;
+		    sv_player->v.armorvalue = v;
+		    sv_player->v.items = sv_player->v.items -
+					((int)(sv_player->v.items) & (int)(IT_ARMOR1 | IT_ARMOR2 | IT_ARMOR3)) +
+					IT_ARMOR1;
 		}
 		break;
-	//johnfitz
-    }
+		//johnfitz
+	}
 
 	//johnfitz -- update currentammo to match new ammo (so statusbar updates correctly)
 	switch ((int)(sv_player->v.weapon))
@@ -2050,7 +2099,7 @@ void Host_Viewframe_f (void)
 
 	f = atoi(Cmd_Argv(1));
 	if (f >= m->numframes)
-		f = m->numframes-1;
+		f = m->numframes - 1;
 
 	e->v.frame = f;
 }
@@ -2133,11 +2182,7 @@ void Host_Startdemos_f (void)
 	int		i, c;
 
 	if (cls.state == ca_dedicated)
-	{
-		if (!sv.active)
-			Cbuf_AddText ("map start\n");
 		return;
-	}
 
 	c = Cmd_Argc() - 1;
 	if (c > MAX_DEMOS)
@@ -2147,7 +2192,7 @@ void Host_Startdemos_f (void)
 	}
 	Con_Printf ("%i demo(s) in loop\n", c);
 
-	for (i=1 ; i<c+1 ; i++)
+	for (i = 1; i < c + 1; i++)
 		strncpy (cls.demos[i-1], Cmd_Argv(i), sizeof(cls.demos[0])-1);
 
 	if (!sv.active && cls.demonum != -1 && !cls.demoplayback)
@@ -2156,7 +2201,7 @@ void Host_Startdemos_f (void)
 		if (!fitzmode)
 		{  /* QuakeSpasm customization: */
 			/* go straight to menu, no CL_NextDemo */
-			Cbuf_InsertText("menu_main");
+			Cbuf_InsertText("menu_main\n");
 			return;
 		}
 		CL_NextDemo ();
@@ -2214,6 +2259,7 @@ void Host_InitCommands (void)
 	Cmd_AddCommand ("maps", Host_Maps_f); //johnfitz
 	Cmd_AddCommand ("game", Host_Game_f); //johnfitz
 	Cmd_AddCommand ("mods", Host_Mods_f); //johnfitz
+	Cmd_AddCommand ("games", Host_Mods_f); // as an alias to "mods" -- S.A. / QuakeSpasm
 	Cmd_AddCommand ("mapname", Host_Mapname_f); //johnfitz
 
 	Cmd_AddCommand ("status", Host_Status_f);
@@ -2256,3 +2302,4 @@ void Host_InitCommands (void)
 
 	Cmd_AddCommand ("mcache", Mod_Print);
 }
+
