@@ -27,8 +27,11 @@ key up events are sent even if in console mode
 
 */
 
-#define		MAXCMDLINE	256
-char	key_lines[32][MAXCMDLINE];
+#define		HISTORY_FILE_NAME "history.txt"
+#define		CMDLINES 32
+
+char		key_lines[CMDLINES][MAXCMDLINE];
+
 int		key_linepos;
 int		shift_down=false;
 int		key_lastpress;
@@ -53,7 +56,7 @@ qboolean	repeatkeys[256]; //johnfitz -- if true, autorepeat is enabled for this
 
 typedef struct
 {
-	char	*name;
+	const char	*name;
 	int		keynum;
 } keyname_t;
 
@@ -165,205 +168,6 @@ keyname_t keynames[] =
 	{NULL,0}
 };
 
-										//  dedicated before exiting
-byte key_map[SDLK_LAST];
-
-void BuildKeyMaps (void)
-{
-    int i;
-
-    for (i = 0; i < SDLK_LAST; i++)
-        key_map[i] = 0;
-
-	key_map[SDLK_BACKSPACE]     = K_BACKSPACE;
-	key_map[SDLK_TAB]           = K_TAB;
-	key_map[SDLK_RETURN]        = K_ENTER;
-	key_map[SDLK_PAUSE]         = K_PAUSE;
-	key_map[SDLK_ESCAPE]        = K_ESCAPE;
-	key_map[SDLK_SPACE]         = K_SPACE;
-	key_map[SDLK_EXCLAIM]       = '!';
-	key_map[SDLK_QUOTEDBL]      = '"';
-	key_map[SDLK_HASH]          = '#';
-	key_map[SDLK_DOLLAR]        = '$';
-	key_map[SDLK_AMPERSAND]     = '&';
-	key_map[SDLK_QUOTE]         = '\'';
-	key_map[SDLK_LEFTPAREN]     = '(';
-	key_map[SDLK_RIGHTPAREN]    = ')';
-	key_map[SDLK_ASTERISK]      = '*';
-	key_map[SDLK_PLUS]          = '+';
-	key_map[SDLK_COMMA]         = ',';
-	key_map[SDLK_MINUS]         = '-';
-	key_map[SDLK_PERIOD]        = '.';
-	key_map[SDLK_SLASH]         = '/';
-                                
-	key_map[SDLK_0]             = '0';
-	key_map[SDLK_1]             = '1';
-	key_map[SDLK_2]             = '2';
-	key_map[SDLK_3]             = '3';
-	key_map[SDLK_4]             = '4';
-	key_map[SDLK_5]             = '5';
-	key_map[SDLK_6]             = '6';
-	key_map[SDLK_7]             = '7';
-	key_map[SDLK_8]             = '8';
-	key_map[SDLK_9]             = '9';
-                                
-	key_map[SDLK_COLON]         = ':';
-	key_map[SDLK_SEMICOLON]     = ';';
-	key_map[SDLK_LESS]          = '<';
-	key_map[SDLK_EQUALS]        = '=';
-	key_map[SDLK_GREATER]       = '>';
-	key_map[SDLK_QUESTION]      = '?';
-	key_map[SDLK_AT]            = '@';
-	key_map[SDLK_LEFTBRACKET]   = '[';
-	key_map[SDLK_BACKSLASH]     = '\\';
-	key_map[SDLK_RIGHTBRACKET]  = ']';
-	key_map[SDLK_CARET]         = '^';
-	key_map[SDLK_UNDERSCORE]    = '_';
-	key_map[SDLK_BACKQUOTE]     = '`';
-
-	key_map[SDLK_a]             = 'a';
-	key_map[SDLK_b]             = 'b';
-	key_map[SDLK_c]             = 'c';
-	key_map[SDLK_d]             = 'd';
-	key_map[SDLK_e]             = 'e';
-	key_map[SDLK_f]             = 'f';
-	key_map[SDLK_g]             = 'g';
-	key_map[SDLK_h]             = 'h';
-	key_map[SDLK_i]             = 'i';
-	key_map[SDLK_j]             = 'j';
-	key_map[SDLK_k]             = 'k';
-	key_map[SDLK_l]             = 'l';
-	key_map[SDLK_m]             = 'm';
-	key_map[SDLK_n]             = 'n';
-	key_map[SDLK_o]             = 'o';
-	key_map[SDLK_p]             = 'p';
-	key_map[SDLK_q]             = 'q';
-	key_map[SDLK_r]             = 'r';
-	key_map[SDLK_s]             = 's';
-	key_map[SDLK_t]             = 't';
-	key_map[SDLK_u]             = 'u';
-	key_map[SDLK_v]             = 'v';
-	key_map[SDLK_w]             = 'w';
-	key_map[SDLK_x]             = 'x';
-	key_map[SDLK_y]             = 'y';
-	key_map[SDLK_z]             = 'z';
-                                
-	key_map[SDLK_DELETE]        = K_DEL;
-                                
-	key_map[SDLK_KP0]           = KP_INS;
-	key_map[SDLK_KP1]           = KP_END;
-	key_map[SDLK_KP2]           = KP_DOWNARROW;
-	key_map[SDLK_KP3]           = KP_PGDN;
-	key_map[SDLK_KP4]           = KP_LEFTARROW;
-	key_map[SDLK_KP5]           = KP_5;
-	key_map[SDLK_KP6]           = KP_RIGHTARROW;
-	key_map[SDLK_KP7]           = KP_HOME;
-	key_map[SDLK_KP8]           = KP_UPARROW;
-	key_map[SDLK_KP9]           = KP_PGUP;
-	key_map[SDLK_KP_PERIOD]     = KP_DEL;
-	key_map[SDLK_KP_DIVIDE]     = KP_SLASH;
-	key_map[SDLK_KP_MULTIPLY]   = KP_STAR;
-	key_map[SDLK_KP_MINUS]      = KP_MINUS;
-	key_map[SDLK_KP_PLUS]       = KP_PLUS;
-	key_map[SDLK_KP_ENTER]      = KP_ENTER;
-	key_map[SDLK_KP_EQUALS]     = 0;
-                                
-	key_map[SDLK_UP]            = K_UPARROW;
-	key_map[SDLK_DOWN]          = K_DOWNARROW;
-	key_map[SDLK_RIGHT]         = K_RIGHTARROW;
-	key_map[SDLK_LEFT]          = K_LEFTARROW;
-	key_map[SDLK_INSERT]        = K_INS;
-	key_map[SDLK_HOME]          = K_HOME;
-	key_map[SDLK_END]           = K_END;
-	key_map[SDLK_PAGEUP]        = K_PGUP;
-	key_map[SDLK_PAGEDOWN]      = K_PGDN;
-                                
-	key_map[SDLK_F1]            = K_F1;
-	key_map[SDLK_F2]            = K_F2;
-	key_map[SDLK_F3]            = K_F3;
-	key_map[SDLK_F4]            = K_F4;
-	key_map[SDLK_F5]            = K_F5;
-	key_map[SDLK_F6]            = K_F6;
-	key_map[SDLK_F7]            = K_F7;
-	key_map[SDLK_F8]            = K_F8;
-	key_map[SDLK_F9]            = K_F9;
-	key_map[SDLK_F10]           = K_F10;
-	key_map[SDLK_F11]           = K_F11;
-	key_map[SDLK_F12]           = K_F12;
-	key_map[SDLK_F13]           = 0;
-	key_map[SDLK_F14]           = 0;
-	key_map[SDLK_F15]           = 0;
-                                
-	key_map[SDLK_NUMLOCK]       = KP_NUMLOCK;
-	key_map[SDLK_CAPSLOCK]      = 0;
-	key_map[SDLK_SCROLLOCK]     = 0;
-	key_map[SDLK_RSHIFT]        = K_SHIFT;
-	key_map[SDLK_LSHIFT]        = K_SHIFT;
-	key_map[SDLK_RCTRL]         = K_CTRL;
-	key_map[SDLK_LCTRL]         = K_CTRL;
-	key_map[SDLK_RALT]          = K_ALT;
-	key_map[SDLK_LALT]          = K_ALT;
-	key_map[SDLK_RMETA]         = 0;
-	key_map[SDLK_LMETA]         = 0;
-	key_map[SDLK_LSUPER]        = 0;
-	key_map[SDLK_RSUPER]        = 0;
-	key_map[SDLK_MODE]          = 0;
-	key_map[SDLK_COMPOSE]       = 0;
-	key_map[SDLK_HELP]          = 0;
-	key_map[SDLK_PRINT]         = 0;
-	key_map[SDLK_SYSREQ]        = 0;
-	key_map[SDLK_BREAK]         = 0;
-	key_map[SDLK_MENU]          = 0;
-	key_map[SDLK_POWER]         = 0;
-	key_map[SDLK_EURO]          = 0;
-	key_map[SDLK_UNDO]          = 0;
-}
-
-/*
-=======
-MapKey
-
-Map from SDL to quake keynums
-=======
-*/
-int Key_Map (SDL_KeyboardEvent *event)
-{
-    return key_map[(*event).keysym.sym];
-
-    /* TODO: keypad handling
-	if (cl_keypad.value) {
-		if (extended) {
-			switch (key) {
-				case K_ENTER:		return KP_ENTER;
-				case '/':			return KP_SLASH;
-				case K_PAUSE:		return KP_NUMLOCK;
-			};
-		} else {
-			switch (key) {
-				case K_HOME:		return KP_HOME;
-				case K_UPARROW:		return KP_UPARROW;
-				case K_PGUP:		return KP_PGUP;
-				case K_LEFTARROW:	return KP_LEFTARROW;
-				case K_RIGHTARROW:	return KP_RIGHTARROW;
-				case K_END:			return KP_END;
-				case K_DOWNARROW:	return KP_DOWNARROW;
-				case K_PGDN:		return KP_PGDN;
-				case K_INS:			return KP_INS;
-				case K_DEL:			return KP_DEL;
-			}
-		}
-	} else {
-		// cl_keypad 0, compatibility mode
-		switch (key) {
-			case KP_STAR:	return '*';
-			case KP_MINUS:	return '-';
-			case KP_5:		return '5';
-			case KP_PLUS:	return '+';
-		}
-	}
-    */
-}
-
 /*
 ==============================================================================
 
@@ -383,6 +187,7 @@ void Key_Console (int key)
 {
 	extern	int con_vislines;
 	extern	char key_tabpartial[MAXCMDLINE];
+	static  char current[MAXCMDLINE]="";
 
 	switch (key)
 	{
@@ -392,7 +197,12 @@ void Key_Console (int key)
 		Cbuf_AddText (key_lines[edit_line]+1);	// skip the prompt
 		Cbuf_AddText ("\n");
 		Con_Printf ("%s\n",key_lines[edit_line]);
-		edit_line = (edit_line + 1) & 31;
+
+		// If the last two lines are identical, skip storing this line in history 
+		// by not incrementing edit_line
+		if (strcmp(key_lines[edit_line],key_lines[(edit_line-1)&31]))
+			edit_line = (edit_line + 1) & 31;
+
 		history_line = edit_line;
 		key_lines[edit_line][0] = ']';
 		key_lines[edit_line][1] = 0; //johnfitz -- otherwise old history items show up in the new edit line
@@ -458,14 +268,16 @@ void Key_Console (int key)
 		return;
 
 	case K_PGUP:
-	//case K_MWHEELUP:
+	// Mouse events never reach the console, especially in windowed mode
+	// when mouse is released to the window manager
+	// case K_MWHEELUP:
 		con_backscroll += keydown[K_CTRL] ? ((con_vislines>>3) - 4) : 2;
 		if (con_backscroll > con_totallines - (vid.height>>3) - 1)
 			con_backscroll = con_totallines - (vid.height>>3) - 1;
 		return;
 
 	case K_PGDN:
-	//case K_MWHEELDOWN:
+	// case K_MWHEELDOWN:
 		con_backscroll -= keydown[K_CTRL] ? ((con_vislines>>3) - 4) : 2;
 		if (con_backscroll < 0)
 			con_backscroll = 0;
@@ -497,14 +309,15 @@ void Key_Console (int key)
 		return;
 
 	case K_UPARROW:
+		// Needs rewriting as we have persistent history
+
+		// if (history_line == 0) return;
+		if (history_line == edit_line) {
+			Q_strcpy(current,key_lines[edit_line]);
+		};
 		key_tabpartial[0] = 0;
-		do
-		{
-			history_line = (history_line - 1) & 31;
-		} while (history_line != edit_line
-				&& !key_lines[history_line][1]);
-		if (history_line == edit_line)
-			history_line = (edit_line+1)&31;
+		history_line = (history_line - 1) & 31;
+
 		Q_strcpy(key_lines[edit_line], key_lines[history_line]);
 		key_linepos = Q_strlen(key_lines[edit_line]);
 		return;
@@ -512,29 +325,29 @@ void Key_Console (int key)
 	case K_DOWNARROW:
 		key_tabpartial[0] = 0;
 
+		if (history_line == edit_line) 
+			return;
+
+		history_line = (history_line + 1) & 31;
+
 		if (history_line == edit_line)
-		{
-			//clear editline
-			key_lines[edit_line][1] = 0;
+			Q_strcpy(key_lines[edit_line], current);
+		else
+			Q_strcpy(key_lines[edit_line], key_lines[history_line]);
+		key_linepos = Q_strlen(key_lines[edit_line]);
+		return;
+
+	case 'c':
+		if (keydown[K_CTRL]) {
+			// Control+C (S.A)
+			//   abort this line.
+			Con_Printf ("%s\n",key_lines[edit_line]);
+			key_lines[edit_line][0] = ']';
+			key_lines[edit_line][1] = 0; //johnfitz -- otherwise old history items show up in the new edit line
 			key_linepos = 1;
+			history_line=edit_line;
 			return;
 		}
-
-		do {
-			history_line = (history_line + 1) & 31;
-		} while (history_line != edit_line
-			&& !key_lines[history_line][1]);
-		if (history_line == edit_line)
-		{
-			key_lines[edit_line][0] = ']';
-			key_linepos = 1;
-		}
-		else
-		{
-			Q_strcpy(key_lines[edit_line], key_lines[history_line]);
-			key_linepos = Q_strlen(key_lines[edit_line]);
-		}
-		return;
 	}
 
 //johnfitz -- clipboard pasting, stolen from zquake
@@ -605,7 +418,7 @@ void Key_Console (int key)
 
 //============================================================================
 
-char chat_buffer[32];
+char chat_buffer[MAXCMDLINE];
 qboolean team_message = false;
 
 void Key_Message (int key)
@@ -648,7 +461,7 @@ void Key_Message (int key)
 		return;
 	}
 
-	if (chat_bufferlen == 31)
+	if (chat_bufferlen == sizeof(chat_buffer)-1)
 		return; // all full
 
 	chat_buffer[chat_bufferlen++] = key;
@@ -667,7 +480,7 @@ the given string.  Single ascii characters return themselves, while
 the K_* names are matched up.
 ===================
 */
-int Key_StringToKeynum (char *str)
+int Key_StringToKeynum (const char *str)
 {
 	keyname_t	*kn;
 
@@ -693,7 +506,7 @@ given keynum.
 FIXME: handle quote special (general escape sequence?)
 ===================
 */
-char *Key_KeynumToString (int keynum)
+const char *Key_KeynumToString (int keynum)
 {
 	keyname_t	*kn;
 	static	char	tinystr[2];
@@ -720,9 +533,9 @@ char *Key_KeynumToString (int keynum)
 Key_SetBinding
 ===================
 */
-void Key_SetBinding (int keynum, char *binding)
+void Key_SetBinding (int keynum, const char *binding)
 {
-	char	*new;
+	char	*new_binding;
 	int		l;
 
 	if (keynum == -1)
@@ -737,10 +550,10 @@ void Key_SetBinding (int keynum, char *binding)
 
 // allocate memory for new binding
 	l = Q_strlen (binding);
-	new = Z_Malloc (l+1);
-	Q_strcpy (new, binding);
-	new[l] = 0;
-	keybindings[keynum] = new;
+	new_binding = (char *) Z_Malloc (l+1);
+	Q_strcpy (new_binding, binding);
+	new_binding[l] = 0;
+	keybindings[keynum] = new_binding;
 }
 
 /*
@@ -860,6 +673,63 @@ void Key_WriteBindings (FILE *f)
 }
 
 
+void History_Init (void)
+{
+	int i, c;
+	FILE *hf;
+
+	for (i = 0; i < CMDLINES; i++)
+	{
+		key_lines[i][0] = ']';
+		key_lines[i][1] = 0;
+	}
+	key_linepos = 1;
+
+//	if (cl_savehistory.value)
+	if ((hf = fopen(HISTORY_FILE_NAME, "rt")) != NULL)
+	{
+		do
+		{
+			i = 1;
+			do
+			{
+				c = fgetc(hf);
+				key_lines[edit_line][i++] = c;
+			} while (c != '\n' && c != EOF && i < MAXCMDLINE);
+			key_lines[edit_line][i - 1] = 0;
+			edit_line = (edit_line + 1) & (CMDLINES - 1);
+		} while (c != EOF && edit_line < CMDLINES);
+		fclose(hf);
+
+		history_line = edit_line = (edit_line - 1) & (CMDLINES - 1);
+		key_lines[edit_line][0] = ']';
+		key_lines[edit_line][1] = 0;
+	}
+}
+
+void History_Shutdown (void)
+{
+	int i;
+	FILE *hf;
+
+//	if (cl_savehistory.value)
+	if ((hf = fopen(HISTORY_FILE_NAME, "wt")) != NULL)
+	{
+		i = edit_line;
+		do
+		{
+			i = (i + 1) & (CMDLINES - 1);
+		} while (i != edit_line && !key_lines[i][1]);
+
+		while (i != edit_line && key_lines[i][1])
+		{
+			fprintf(hf, "%s\n", key_lines[i] + 1);
+			i = (i + 1) & (CMDLINES - 1);
+		}
+		fclose(hf);
+	}
+}
+
 /*
 ===================
 Key_Init
@@ -869,14 +739,18 @@ void Key_Init (void)
 {
 	int		i;
 
-    BuildKeyMaps();
+	BuildKeyMaps();
 
-	for (i=0 ; i<32 ; i++)
+	History_Init ();
+
+#if 0 /* This section of code is now done in History_Init */
+	for (i = 0; i < 32; i++)
 	{
 		key_lines[i][0] = ']';
 		key_lines[i][1] = 0;
 	}
 	key_linepos = 1;
+#endif
 
 	key_blinktime = realtime; //johnfitz
 
@@ -1057,7 +931,7 @@ void Key_Event (int key, qboolean down)
 //
 // during demo playback, most keys bring up the main menu
 //
-	if (cls.demoplayback && down && consolekeys[key] && key_dest == key_game)
+	if (cls.demoplayback && down && consolekeys[key] && key_dest == key_game && key != K_TAB)
 	{
 		M_ToggleMenu_f ();
 		return;
